""" Testing D transform implementation
"""

from operator import add
import numpy as np

from regreg.affine import (broadcast_first, 
                           affine_transform, 
                           linear_transform,
                           AffineError, 
                           composition, 
                           adjoint,
                           astransform,
                           reshape,
                           selector,
                           vstack,
                           hstack,
                           product,
                           power_L,
                           posneg,
                           scalar_multiply,
                           todense)

from numpy.testing import (assert_array_almost_equal,
                           assert_array_equal)

import scipy.sparse
from nose.tools import assert_true, assert_equal, assert_raises

import regreg.api as rr
from regreg.tests.decorators import set_seed_for_test

def test_broad_first():
    # Test broadcasting over second axis
    a = np.arange(4) + 10
    b = np.arange(4).reshape(4,1)
    c = broadcast_first(a, b, add)
    res = a[:,None] + b
    assert_equal(res.shape, c.shape)
    assert_array_almost_equal(c, res)
    res1d = res.ravel()
    c = broadcast_first(b, a, add)
    assert_equal(res1d.shape, c.shape)
    assert_array_almost_equal(c, res1d)
    c = broadcast_first(a, b.ravel(), add)
    assert_equal(res1d.shape, c.shape)
    assert_array_almost_equal(c, res1d)

def test_dot():

    A = np.random.standard_normal((20, 12))
    B = np.random.standard_normal((10, 20))

    C = rr.astransform(B).dot(rr.astransform(A))

    assert_equal(C.shape, (10, 12))
    Z = np.random.standard_normal(12)
    assert_array_almost_equal(C.dot(Z), B.dot(A.dot(Z)))

    assert_equal(C.ndim, 2)

def test_affine_transform():
    # Test affine transform

    m = 20
    x1d = np.arange(m)
    x2d = x1d[:,None]
    x22d = np.c_[x2d, x2d]
    # Error if both of linear and affine components are None
    assert_raises(AffineError, affine_transform, None, None)
    assert_raises(AffineError, linear_transform, None)
    # With linear None and 0 affine offset - identity transform
    for x in (x1d, x2d, x22d):
        trans = affine_transform(None, np.zeros((m,1)))
        assert_array_almost_equal(trans.affine_map(x), x)
        assert_array_almost_equal(trans.linear_map(x), x)
        assert_array_almost_equal(trans.dot(x), x)
        assert_array_almost_equal(trans.adjoint_map(x), x)
        assert_array_almost_equal(trans.T.dot(x), x)
        # With linear eye and None affine offset - identity again
        trans = affine_transform(np.eye(m), None)
        assert_array_almost_equal(trans.affine_map(x), x)
        assert_array_almost_equal(trans.linear_map(x), x)
        assert_array_almost_equal(trans.adjoint_map(x), x)
        # affine_transform as input
        trans = affine_transform(trans, None)
        assert_array_almost_equal(trans.affine_map(x), x)
        assert_array_almost_equal(trans.linear_map(x), x)
        assert_array_almost_equal(trans.adjoint_map(x), x)
        # diag
        trans = affine_transform(np.ones(m), None, True)
        assert_array_almost_equal(trans.affine_map(x), x)
        assert_array_almost_equal(trans.linear_map(x), x)
        assert_array_almost_equal(trans.adjoint_map(x), x)

@set_seed_for_test()
def test_composition():
    X1 = np.random.standard_normal((20,30))
    X2 = np.random.standard_normal((30,10))
    b1 = np.random.standard_normal(20)
    b2 = np.random.standard_normal(30)
    L1 = affine_transform(X1, b1)
    L2 = affine_transform(X2, b2)

    z = np.random.standard_normal(10)
    w = np.random.standard_normal(20)
    comp = composition(L1,L2)

    assert_array_almost_equal(comp.linear_map(z), np.dot(X1, np.dot(X2, z)))
    assert_array_almost_equal(comp.adjoint_map(w), np.dot(X2.T, np.dot(X1.T, w)))
    assert_array_almost_equal(comp.affine_map(z), np.dot(X1, np.dot(X2, z)+b2)+b1)

@set_seed_for_test()
def test_composition2():
    X1 = np.random.standard_normal((20,30))
    X2 = np.random.standard_normal((30,10))
    X3 = np.random.standard_normal((10,20))

    b1 = np.random.standard_normal(20)
    b2 = np.random.standard_normal(30)
    b3 = np.random.standard_normal(10)

    L1 = affine_transform(X1, b1)
    L2 = affine_transform(X2, b2)
    L3 = affine_transform(X3, b3)

    z = np.random.standard_normal(20)
    w = np.random.standard_normal(20)
    comp = composition(L1,L2,L3)

    assert_array_almost_equal(comp.linear_map(z), 
                       np.dot(X1, np.dot(X2, np.dot(X3, z))))
    assert_array_almost_equal(comp.adjoint_map(w), 
                       np.dot(X3.T, np.dot(X2.T, np.dot(X1.T, w))))
    assert_array_almost_equal(
        comp.affine_map(z),
        np.dot(X1, np.dot(X2, np.dot(X3, z) + b3) + b2) + b1)

@set_seed_for_test()
def test_adjoint():
    X = np.random.standard_normal((20,30))
    b = np.random.standard_normal(20)
    L = affine_transform(X, b)

    z = np.random.standard_normal(30)
    w = np.random.standard_normal(20)
    A = adjoint(L)

    assert_array_almost_equal(A.linear_map(w), L.adjoint_map(w))
    assert_array_almost_equal(A.affine_map(w), L.adjoint_map(w))
    assert_array_almost_equal(A.adjoint_map(z), L.linear_map(z))

@set_seed_for_test()
def test_affine_sum():

    n = 100
    p = 25

    X1 = np.random.standard_normal((n,p))
    X2 = np.random.standard_normal((n,p))
    b = np.random.standard_normal(n)
    v = np.random.standard_normal(p)

    transform1 = rr.affine_transform(X1, b)
    transform2 = rr.linear_transform(X2)
    sum_transform = rr.affine_sum([transform1, transform2])

    yield assert_array_almost_equal, np.dot(X1,v) + np.dot(X2,v) + b, sum_transform.affine_map(v)
    yield assert_array_almost_equal, np.dot(X1,v) + np.dot(X2,v), sum_transform.linear_map(v)
    yield assert_array_almost_equal, np.dot(X1.T,b) + np.dot(X2.T,b), sum_transform.adjoint_map(b)
    yield assert_array_almost_equal, b, sum_transform.affine_offset

    sum_transform = rr.affine_sum([transform1, transform2], weights=[3,4])

    yield assert_array_almost_equal, 3*(np.dot(X1,v) + b) + 4*(np.dot(X2,v)), sum_transform.affine_map(v)
    yield assert_array_almost_equal, 3*np.dot(X1,v) + 4*np.dot(X2,v), sum_transform.linear_map(v)
    yield assert_array_almost_equal, 3*np.dot(X1.T,b) + 4*np.dot(X2.T,b), sum_transform.adjoint_map(b)
    yield assert_array_almost_equal, 3*b, sum_transform.affine_offset

@set_seed_for_test()
def test_affine_sparse():
    # test using sparse matrices for affine transforms

    n = 100
    p = 25

    X1 = scipy.sparse.csr_matrix(np.random.standard_normal((n,p)))
    b = scipy.sparse.csr_matrix(np.random.standard_normal(n))
    v = np.random.standard_normal(p)
    y = np.random.standard_normal(n)

    transform1 = rr.affine_transform(X1, b)

    transform1.linear_map(v)
    transform1.adjoint_map(y)
    transform1.affine_map(v)
    
    # should raise a warning about type of sparse matrix

    X1 = scipy.sparse.coo_matrix(np.random.standard_normal((n,p)))
    b = scipy.sparse.coo_matrix(np.random.standard_normal(n))
    v = np.random.standard_normal(p)
    y = np.random.standard_normal(n)

    transform2 = rr.affine_transform(X1, b)

    transform2.linear_map(v)
    transform2.adjoint_map(y)
    transform2.affine_map(v)
    
@set_seed_for_test()
def test_row_matrix():
    # make sure we can input a vector as a transform

    n, p = 20, 1
    x = np.random.standard_normal(n)
    b = np.random.standard_normal(p)
    v = np.random.standard_normal(n)
    y = np.random.standard_normal(p)

    transform1 = rr.linear_transform(x)
    transform2 = rr.affine_transform(x, b)

    transform1.linear_map(v)
    transform1.affine_map(v)
    transform1.adjoint_map(y)

    transform2.linear_map(v)
    transform2.affine_map(v)
    transform2.adjoint_map(y)

@set_seed_for_test()
def test_coefs_matrix():

    n, p, q = 20, 10, 5

    X = np.random.standard_normal((n, p))
    B = np.random.standard_normal((n, q))
    V = np.random.standard_normal((p, q))
    Y = np.random.standard_normal((n, q))

    transform1 = rr.linear_transform(X, input_shape=(p,q))
    assert_equal(transform1.linear_map(V).shape, (n,q))
    assert_equal(transform1.affine_map(V).shape, (n,q))
    assert_equal(transform1.adjoint_map(Y).shape, (p,q))

    transform2 = rr.affine_transform(X, B, input_shape=(p,q))
    assert_equal(transform2.linear_map(V).shape, (n,q))
    assert_equal(transform2.affine_map(V).shape, (n,q))
    assert_equal(transform2.adjoint_map(Y).shape, (p,q))

def test_selector():
    X = np.arange(30).reshape((6,5))
    offset = np.arange(6)
    transform = affine_transform(X, offset)
    apply_to_first5 = selector(slice(0,5), (20,), transform)
    apply_to_first5.affine_map(np.arange(20))
    apply_to_first5.linear_map(np.arange(20))
    apply_to_first5.adjoint_map(np.arange(6))

    just_select = selector(slice(0,5), (20,))
    just_select.affine_map(np.arange(20))
    just_select.linear_map(np.arange(20))
    just_select.adjoint_map(np.arange(5))
    np.testing.assert_allclose(np.arange(5), just_select.linear_map(np.arange(20)))

def test_reshape():
    reshape_ = reshape((30,), (6,5))
    assert_equal(reshape_.linear_map(np.arange(30)).shape, (6,5))
    assert_equal(reshape_.affine_map(np.arange(30)).shape, (6,5))
    assert_equal(reshape_.adjoint_map(np.zeros((6,5))).shape, (30,))

@set_seed_for_test()
def test_stack_product():
    X = np.random.standard_normal((5, 30))
    Y = np.random.standard_normal((5, 30))
    Z = np.random.standard_normal((5, 31))
    U = np.random.standard_normal((6, 30))
    stack = vstack([X, Y])

    assert_raises(ValueError, vstack, [X, Z])
    assert_raises(ValueError, hstack, [X, U])

    np.testing.assert_allclose(stack.linear_map(np.arange(30))[:5], np.dot(X, np.arange(30)))
    np.testing.assert_allclose(stack.linear_map(np.arange(30))[5:], np.dot(Y, np.arange(30)))

    np.testing.assert_allclose(stack.affine_map(np.arange(30))[:5], np.dot(X, np.arange(30)))
    np.testing.assert_allclose(stack.affine_map(np.arange(30))[5:], np.dot(Y, np.arange(30)))

    np.testing.assert_allclose(stack.adjoint_map(np.arange(10)), np.dot(X.T, np.arange(5)) + np.dot(Y.T, np.arange(5, 10)))

    _hstack = hstack([X, Y, Z])
    _hstack.linear_map(np.arange(91))
    _hstack.affine_map(np.arange(91))
    _hstack.adjoint_map(np.arange(5))

    b = np.random.standard_normal(5)
    XA = rr.affine_transform(X, b)
    _product = product([XA,Y])
    np.testing.assert_allclose(_product.linear_map(np.arange(60))[:5], np.dot(X, np.arange(30)))
    np.testing.assert_allclose(_product.linear_map(np.arange(60))[5:], np.dot(Y, np.arange(30, 60)))
    np.testing.assert_allclose(_product.affine_map(np.arange(60))[:5], np.dot(X, np.arange(30)) + b)
    np.testing.assert_allclose(_product.affine_map(np.arange(60))[5:], np.dot(Y, np.arange(30, 60)))

    np.testing.assert_allclose(_product.adjoint_map(np.arange(10))[:30], np.dot(X.T, np.arange(5)))
    np.testing.assert_allclose(_product.adjoint_map(np.arange(10))[30:], np.dot(Y.T, np.arange(5, 10)))

    scale_prod = scalar_multiply(_product, 2)
    np.testing.assert_allclose(scale_prod.linear_map(np.arange(60)), 2 * _product.linear_map(np.arange(60)))
    np.testing.assert_allclose(scale_prod.affine_map(np.arange(60)), 2 * _product.affine_map(np.arange(60)))
    np.testing.assert_allclose(scale_prod.adjoint_map(np.arange(60)), 2 * _product.adjoint_map(np.arange(60)))

@set_seed_for_test()
def test_posneg():

    X = np.random.standard_normal((40, 5))
    X_pn = posneg(X)
    
    V = np.random.standard_normal((2, 5))
    U = np.random.standard_normal(40)
    np.testing.assert_allclose(X_pn.linear_map(V), np.dot(X, V[0] - V[1]))
    np.testing.assert_allclose(X_pn.affine_map(V), np.dot(X, V[0] - V[1]))
    np.testing.assert_allclose(X_pn.adjoint_map(U)[0], np.dot(X.T, U))
    np.testing.assert_allclose(X_pn.adjoint_map(U)[1], -np.dot(X.T, U))

@set_seed_for_test()
def test_misc():
    X = np.random.standard_normal((40, 5))
    power_L(X)
    Xa = rr.astransform(X)
    np.testing.assert_allclose(todense(Xa), X)

    reshapeA = adjoint(reshape((30,), (6,5)))
    assert_raises(NotImplementedError, todense, reshapeA)



"""
Implementation of the SLOPE proximal operator of

https://statweb.stanford.edu/~candes/papers/SLOPE.pdf

"""
from warnings import warn
from copy import copy
import numpy as np
from scipy import sparse

try:
     from sklearn.isotonic import isotonic_regression as isotonic_regression_sk
     have_sklearn_iso = True
except ImportError:
     warn('unable to import isotonic regression from sklearn, using a pure python implementation')
     have_sklearn_iso = False
from ._isotonic_py import pav as isotonic_regression_py

from .seminorms import seminorm

from ..atoms import _work_out_conjugate
from ..objdoctemplates import objective_doc_templater
from ..doctemplates import (doc_template_user, doc_template_provider)

@objective_doc_templater()
class slope(seminorm):

    """
    The SLOPE penalty
    """

    use_sklearn = have_sklearn_iso

    objective_template = r"""\sum_j \lambda_j |%(var)s_{(j)}|"""
    objective_vars = seminorm.objective_vars.copy()
    objective_vars['normklass'] = 'slope'
    objective_vars['dualnormklass'] = 'slope_conjugate'

    def __init__(self, weights, 
                 lagrange=None, 
                 bound=None, 
                 offset=None, 
                 quadratic=None,
                 initial=None):

         weights = np.array(weights, np.float)
         if not np.allclose(-weights, np.sort(-weights)):
              raise ValueError('weights should be non-increasing')
         if not np.all(weights > 0):
              raise ValueError('weights must be positive')
         
         self.weights = weights
         self._dummy = np.arange(self.weights.shape[0])
         
         seminorm.__init__(self, self.weights.shape,
                           lagrange=lagrange,
                           bound=bound,
                           quadratic=quadratic,
                           initial=initial,
                           offset=offset)

    @doc_template_user
    def seminorm(self, x, lagrange=None, check_feasibility=False):
         lagrange = seminorm.seminorm(self, x, 
                                      check_feasibility=check_feasibility, 
                                      lagrange=lagrange)
         xsort = np.sort(np.fabs(x))[::-1]
         return lagrange * np.fabs(xsort * self.weights).sum()

    @doc_template_user
    def constraint(self, x, bound=None):
         bound = seminorm.constraint(self, x, bound=bound)
         inbox = self.seminorm(x, lagrange=1,
                              check_feasibility=True) <= bound * (1 + self.tol)
         if inbox:
              return 0
         else:
              return np.inf

    @doc_template_user
    def lagrange_prox(self, x,  lipschitz=1, lagrange=None):
         lagrange = seminorm.lagrange_prox(self, x, lipschitz, lagrange)
         return _basic_proximal_map(x, self.weights * lagrange / lipschitz, use_sklearn=self.use_sklearn)
    
    @doc_template_user
    def bound_prox(self, x, bound=None):
         raise NotImplementedError

    def __copy__(self):
         return self.__class__(self.weights.copy(),
                               quadratic=self.quadratic,
                               initial=self.coefs,
                               bound=copy(self.bound),
                               lagrange=copy(self.lagrange),
                               offset=copy(self.offset))
    
    def __repr__(self):
         if self.lagrange is not None:
              if not self.quadratic.iszero:
                   return "%s(%s, lagrange=%f, offset=%s)" % \
                       (self.__class__.__name__,
                        str(self.weights),
                        self.lagrange,
                        str(self.offset))
              else:
                   return "%s(%s, lagrange=%f, offset=%s, quadratic=%s)" % \
                       (self.__class__.__name__, 
                        str(self.weights),
                        self.lagrange,
                        str(self.offset),
                        self.quadratic)
         else:
              if not self.quadratic.iszero:
                   return "%s(%s, bound=%f, offset=%s)" % \
                       (self.__class__.__name__,
                        str(self.weights),
                        self.bound,
                        str(self.offset))
              else:
                   return "%s(%s, bound=%f, offset=%s, quadratic=%s)" % \
                       (self.__class__.__name__,
                        str(self.weights),
                        self.bound,
                        str(self.offset),
                        self.quadratic)
              

    @doc_template_user
    def get_conjugate(self):
         if self.quadratic.coef == 0:
              offset, outq = _work_out_conjugate(self.offset, self.quadratic)
              if self.bound is None:
                   cls = conjugate_slope_pairs[self.__class__]
                   atom = cls(self.weights, 
                              bound=self.lagrange, 
                              lagrange=None,
                              offset=offset,
                              quadratic=outq)
              else:
                   cls = conjugate_slope_pairs[self.__class__]
                   atom = cls(self.weights,
                              lagrange=self.bound, 
                              bound=None,
                              offset=offset,
                              quadratic=outq)
         else:
              atom = smooth_conjugate(self)

         self._conjugate = atom
         self._conjugate._conjugate = self
         return self._conjugate
    conjugate = property(get_conjugate)

@objective_doc_templater()
class slope_conjugate(slope):

     r"""
     The dual of the slope penalty:math:`\ell_{\infty}` norm
     """

     objective_template = r"""{\cal D}^{SLOPE}(%(var)s)"""
     objective_vars = seminorm.objective_vars.copy()

     @doc_template_user
     def seminorm(self, x, lagrange=None, check_feasibility=False):
          lagrange = seminorm.seminorm(self, x, 
                                       check_feasibility=check_feasibility, 
                                       lagrange=lagrange)
          xsort = np.sort(np.fabs(x))[::-1]
          xsort_cumsum = np.cumsum(xsort)
          w_cumsum = np.cumsum(self.weights)
          return lagrange * np.fabs(xsort_cumsum / w_cumsum).max()

     @doc_template_user
     def constraint(self, x, bound=None):
          bound = seminorm.constraint(self, x, bound=bound)
          inbox = self.seminorm(x, lagrange=1,
                                check_feasibility=True) <= bound * (1+self.tol)
          if inbox:
               return 0
          else:
               return np.inf

     @doc_template_user
     def lagrange_prox(self, x,  lipschitz=1, lagrange=None):
          raise NotImplementedError

     @doc_template_user
     def bound_prox(self, x, bound=None):
          bound = seminorm.bound_prox(self, x, bound)
          
          # the proximal map is evaluated
          # by working out the SLOPE proximal
          # map and computing the residual
          
          _slope_prox = _basic_proximal_map(x, self.weights * bound, use_sklearn=self.use_sklearn)
          return x - _slope_prox

def _basic_proximal_map(center, weights, use_sklearn=have_sklearn_iso):
     """
     Proximal algorithm described (2.3) of SLOPE
     though sklearn isotonic has ordering reversed.
     """
     
     # the proximal map sorts the absolute values,
     # runs isotonic regression with an offset
     # reassigns the signs
     
     _arg = np.argsort(np.fabs(center))
     shifted_center = np.array(np.fabs(center)[_arg] - weights[::-1]).astype(np.float)
     if use_sklearn:
          _prox_val = np.clip(isotonic_regression_sk(shifted_center), 0, np.inf)
     else:
          _prox_val = np.clip(isotonic_regression_py(shifted_center), 0, np.inf)
     _return_val = np.zeros_like(_prox_val)
     _return_val[_arg] = _prox_val
     _return_val *= np.sign(center)
     return _return_val

conjugate_slope_pairs = {}
for n1, n2 in [(slope, slope_conjugate)]:
     conjugate_slope_pairs[n1] = n2
     conjugate_slope_pairs[n2] = n1



"""
Solving basis pursuit with TFOCS
"""
import numpy as np
import nose.tools as nt

import regreg.api as rr
from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_tfocs():
    n, p = 100, 200
    X = np.random.standard_normal((n, p))
    beta = np.zeros(p)
    beta[:4] = 3
    Y = np.random.standard_normal(n) + np.dot(X, beta)

    lscoef = np.dot(np.linalg.pinv(X), Y)
    minimum_l2 = np.linalg.norm(Y - np.dot(X, lscoef))
    maximum_l2 = np.linalg.norm(Y)

    l2bound = (minimum_l2 + maximum_l2) * 0.5

    constraint = rr.l2norm.affine(X,-Y,bound=l2bound)
    T, conjugate = constraint.dual
    l1 = rr.l1norm(p, lagrange=1)

    primal, dual = rr.tfocs(l1, T, conjugate, coef_tol=1.e-4)
    nt.assert_true(np.fabs(np.linalg.norm(Y - np.dot(X, primal)) - l2bound) <= l2bound * 1.e-3)



from __future__ import print_function, division, absolute_import

from warnings import warn
from copy import copy
import gc

import numpy as np
import numpy.linalg as npl

from scipy.stats import rankdata

from . import subsample_columns
from ..affine import power_L, normalize, astransform
from ..smooth import glm, affine_smooth, sum as smooth_sum
from ..smooth.quadratic import quadratic_loss
from ..problems.simple import simple_problem
from ..identity_quadratic import identity_quadratic as iq
from ..atoms.sparse_group_lasso import (sparse_group_lasso,
                                        _gauge_function_dual_strong,
                                        _inside_set_strong)
from .group_lasso import group_lasso_path, default_lagrange_sequence

class sparse_group_lasso_path(group_lasso_path):

    BIG = 1e12 # lagrange parameter for finding null solution

    def __init__(self, 
                 saturated_loss,
                 X, 
                 groups,
                 lasso_weights,
                 weights={},
                 elastic_net_param=None,
                 alpha=1.,  # elastic net mixing -- 1 is LASSO
                 l1_alpha=0.95, # mix between l1 and l2 penalty
                 ):

        self.saturated_loss = saturated_loss
        self.X = astransform(X)
        self.l1_alpha = l1_alpha # used only in constructor and 
                                 # in subsample

        # the penalty parameters

        self.alpha = alpha
        self.penalty = sparse_group_lasso(groups, lasso_weights, weights=weights, lagrange=1)
        l2_weight = 1 - l1_alpha
        for g in self.penalty.weights.keys():
            self.penalty.set_weight(g, l2_weight * self.penalty.weights[g])
        self.penalty.lasso_weights = self.penalty.lasso_weights * l1_alpha
        self.group_shape = (len(np.unique(self.penalty.groups)),)
        self.shape = self.penalty.shape

        # elastic net part
        if elastic_net_param is None:
            elastic_net_param = np.ones(self.shape)
        self.elastic_net_param = elastic_net_param

        # find lagrange_max

        unpenalized_groups, unpenalized_idx = self.unpenalized

        self.solution = np.zeros(self.penalty.shape)

        self.ever_active_groups = self.updated_ever_active(unpenalized_idx)

        self._unpenalized_vars = _candidate_bool(self.penalty.groups, 
                                                 unpenalized_groups)
        self._penalized_vars = np.zeros(self.shape, np.bool)
        self._penalized_vars[self._unpenalized_vars] = 0

        if np.any(unpenalized_idx):
            (self.final_step, 
             null_grad, 
             null_soln,
             null_linpred,
             _) = self.solve_subproblem(self.solution,
                                        unpenalized_groups,
                                        self.BIG,
                                        tol=1.e-8)
            self.linear_predictor = null_linpred
            self.solution[self._unpenalized_vars] = null_soln
        else:
            self.linear_predictor = np.zeros(self.saturated_loss.shape)

        if np.any(self.elastic_net_param[self._unpenalized_vars]):
            warn('convention is that unpenalized parameters with have no Lagrange parameter in front '
                 'of their ridge term so that lambda_max is easily computed')

        self.grad_solution = (self.full_gradient(self.saturated_loss, 
                                                 self.linear_predictor) + self.enet_grad(self.solution, 
                                                                                         self._penalized_vars,
                                                                                         1))

    @property
    def unpenalized(self):
        """
        Unpenalized groups and variables.

        Returns
        -------

        groups : sequence
            Groups with weights equal to 0.

        variables : ndarray
            Boolean indicator that is True if no penalty on that variable.

        """
        if not hasattr(self, "_unpen_groups"):
            self._unpen_groups = []
            self._unpen_group_idx = np.zeros(self.group_shape, np.bool)
            for i, g in enumerate(self.penalty._sorted_groupids):
                unpen_group = (self.penalty.weights[g] == 0 +
                               np.any(self.penalty.lasso_weights[self.penalty.groups == g]
                                      == 0))
                self._unpen_group_idx[i] = unpen_group
                if unpen_group:
                    self._unpen_groups.append(g)
        return self._unpen_groups, self._unpen_group_idx

    # methods potentially overwritten in subclasses for I/O considerations

    def subsample(self,
                  case_idx):
        '''

        Create a new path, by subsampling
        cases of `self.saturated_loss`.

        Case weights are computed
        with `self.saturated_loss.subsample`.

        Parameters
        ----------

        case_idx : index
            An index-like object used 
            to specify which cases to include
            in the subsample.

        Returns
        -------

        subsample_path : path object
            A path object with a modified smooth part
            reflecting the subsampling.

        '''
        subsample_loss = self.saturated_loss.subsample(case_idx)
        return self.__class__(subsample_loss,
                              self.X,
                              self.penalty.groups,
                              self.penalty.lasso_weights,
                              weights=self.penalty.weights,
                              elastic_net_param=self.elastic_net_param,
                              alpha=self.alpha,
                              l1_alpha=self.l1_alpha)

    def check_KKT(self,
                  grad_solution,
                  solution,
                  lagrange,
                  penalty=None):

        '''

        Check KKT conditions over
        the groups in the path.
        Returns boolean indicating
        which groups are failing the KKT conditions
        (these could be `active` groups or
        `inactive` groups).

        Parameters
        ----------

        grad_solution : ndarray
             Candidate for gradient of smooth loss at 
             Lagrange value `lagrange`.

        solution : ndarray
             Candidate for solution to problem 
             Lagrange value `lagrange`.

        lagrange : float
             Lagrange value for penalty

        penalty : object (optional)
             A sparse group LASSO penalty. If None, defaults
             to `self.penalty`.

        '''
        if penalty is None:
            penalty = self.penalty

        active, inactive_ranks = _check_KKT(penalty,
                                            grad_solution, 
                                            solution, 
                                            lagrange)
        return active > 0, inactive_ranks


    def strong_set(self,
                   lagrange_cur,
                   lagrange_new,
                   grad_solution):

        _strong_bool = _strong_set(self.penalty,
                                   self._unpen_group_idx,
                                   lagrange_cur,
                                   lagrange_new,
                                   grad_solution)
        _strong = np.array([self.penalty._sorted_groupids[i] for i in np.nonzero(_strong_bool)[0]])
        return (_strong, 
                _strong_bool,
                np.nonzero(_candidate_bool(self.penalty.groups, _strong))[0])
                                             
    def solve_subproblem(self, 
                         solution,
                         candidate_groups, 
                         lagrange_new, 
                         **solve_args):
    
        # solve a problem with a candidate set

        sub_loss, sub_penalty, sub_X, candidate_bool = _restricted_problem(self.X, 
                                                                           self.saturated_loss, 
                                                                           self.alpha * lagrange_new, 
                                                                           self.penalty.groups,
                                                                           self.penalty.lasso_weights,
                                                                           self.penalty.weights,
                                                                           candidate_groups,
                                                                           self.subsample_columns)
        if self.alpha < 1:
            sub_elastic_net = self.enet_loss(lagrange_new,
                                             candidate_groups)

            sub_loss = smooth_sum([sub_loss, sub_elastic_net])

        sub_problem = simple_problem(sub_loss, sub_penalty)
        sub_problem.coefs[:] = solution[candidate_bool] # warm start
        sub_soln = sub_problem.solve(**solve_args)
        sub_grad = sub_loss.smooth_objective(sub_soln, mode='grad') 
        sub_linear_pred = sub_X.dot(sub_soln)
        return sub_problem.final_step, sub_grad, sub_soln, sub_linear_pred, candidate_bool

    def restricted_penalty(self, subset):
        if subset is not None:
            groups = self.penalty.groups[subset]
            lasso_weights = self.penalty.lasso_weights[subset]
        else:
            groups = self.penalty.groups
            lasso_weights = self.penalty.lasso_weights
        return sparse_group_lasso(groups,
                                  lasso_weights,
                                  weights=self.penalty.weights,
                                  lagrange=1)#
# Some common loss factories

def logistic(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return sparse_group_lasso_path(glm.logistic_loglike(Y.shape, Y), X, *args, **keyword_args)

def probit(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return sparse_group_lasso_path(glm.probit_loglike(Y.shape, Y), X, *args, **keyword_args)

def cloglog(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return sparse_group_lasso_path(glm.cloglog_loglike(Y.shape, Y), X, *args, **keyword_args)

def gaussian(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return sparse_group_lasso_path(glm.gaussian_loglike(Y.shape, Y), X, *args, **keyword_args)

def cox(X, T, S, *args, **keyword_args):
    T, S = np.asarray(T), np.asarray(S)
    return sparse_group_lasso_path(glm.cox_loglike(T.shape, T, S), X, *args, **keyword_args)

def poisson(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return sparse_group_lasso_path(glm.poisson_loglike(Y.shape, Y), X, *args, **keyword_args)

def huber(X, Y, smoothing_parameter, *args, **keyword_args):
    Y = np.asarray(Y)
    return sparse_group_lasso_path(glm.huber_loss(Y.shape, Y, smoothing_parameter), X, *args, **keyword_args)

def huber_svm(X, Y, smoothing_parameter, *args, **keyword_args):
    Y = np.asarray(Y)
    return sparse_group_lasso_path(glm.huber_svm(Y.shape, Y, smoothing_parameter), X, *args, **keyword_args)

# private functions

def _candidate_bool(groups, candidate_groups):

    candidate_bool = np.zeros(groups.shape, np.bool)
    for g in candidate_groups:
        group = groups == g
        candidate_bool += group

    return candidate_bool

def _strong_set(penalty,
                unpenalized_idx, 
                lagrange_cur,
                lagrange_new,
                gradient,
                slope_estimate=1):

    """
    Return a Boolean indicator for each group
    indicating whether in the strong set or not.
    """

    thresh = (slope_estimate + 1) * lagrange_new - slope_estimate * lagrange_cur
    thresh = (slope_estimate + 1) * lagrange_new - slope_estimate * lagrange_cur
    test = np.zeros(len(penalty._sorted_groupids), np.bool)
    for i, g in enumerate(penalty._sorted_groupids):
        group = penalty.groups == g
        test[i] = _inside_set_strong(gradient[group],
                                     thresh,
                                     penalty.lasso_weights[group],
                                     penalty.weights[g]) == False
    return test

# def _restricted_elastic_net(elastic_net_params, 
#                             penalized,
#                             groups,
#                             lagrange, 
#                             alpha,
#                             candidate_groups):

#     candidate_bool = _candidate_bool(groups, candidate_groups)

#     new_params = elastic_net_params * (1 - alpha)
#     new_params[penalized] *= lagrange 
#     new_params = new_params[candidate_bool]
#     return quadratic_loss(new_params.shape,
#                           new_params,
#                           Qdiag=True)

def _restricted_problem(X, 
                        saturated_loss, 
                        alpha,
                        groups,
                        lasso_weights,
                        group_weights, 
                        candidate_groups,
                        subsample_columns):

    candidate_bool = _candidate_bool(groups, candidate_groups)
    X_candidate = subsample_columns(X, np.nonzero(candidate_bool)[0])
    restricted_penalty = sparse_group_lasso(groups[candidate_bool],
                                            lasso_weights[candidate_bool],
                                            weights=copy(group_weights), # should be OK to reuse weights dict
                                            lagrange=alpha)

    restricted_loss = affine_smooth(saturated_loss, X_candidate)
    return restricted_loss, restricted_penalty, X_candidate, candidate_bool

# for paths

def _check_KKT(sglasso, 
               grad, 
               solution, 
               lagrange, 
               tol=1.e-2):

    """
    Check whether (grad, solution) satisfy
    KKT conditions at a given tolerance.

    Assumes glasso is group_lasso in lagrange form
    so that glasso.lagrange is not None
    """

    terms = sglasso.terms(solution)
    norm_soln = np.linalg.norm(solution)

    ACTIVE_L1 = 10
    ACTIVE_NORM = 11
    ACTIVE_L2 = 12
    INACTIVE = 2
    UNPENALIZED = 3

    active_results = np.zeros(len(sglasso._sorted_groupids), np.int)
    inactive_results = np.zeros(len(sglasso._sorted_groupids), np.float)

    for i, g in enumerate(sglasso._sorted_groupids):
        group = sglasso.groups == g
        subgrad_g = -grad[group]
        l1weight_g = sglasso.lasso_weights[group]
        l2weight_g = sglasso.weights[g]
        soln_g = solution[group]

        if terms[i] > 1.e-6 * norm_soln: # active group
            val_g, l1subgrad_g, l2subgrad_g = _gauge_function_dual_strong(subgrad_g, 
                                                                          l1weight_g,
                                                                          l2weight_g)
            if val_g < lagrange * (1 - tol):
                active_results[i] = ACTIVE_NORM
            nonz = soln_g != 0

            # nonzero coordinates need the right sign and size
            if (np.linalg.norm((l1subgrad_g - l1weight_g * np.sign(soln_g) * lagrange)[nonz]) > 
                tol * max(1, np.linalg.norm(soln_g))):
                active_results[i] = ACTIVE_L1

            # l2 subgrad should be parallel to soln_g
            if np.linalg.norm(l2subgrad_g / np.linalg.norm(l2subgrad_g) - 
                              soln_g / np.linalg.norm(soln_g)) > tol:
                active_results[i] = ACTIVE_L2
        elif l1weight_g.sum() + sglasso.weights[g] > 0: # inactive penalized
            inactive_results[i] = _gauge_function_dual_strong(subgrad_g, l1weight_g,
                                                              l2weight_g)[0] / lagrange
        else:   # unpenalized
            active_results[i] = (np.linalg.norm(subgrad_g) > tol * sqlasso.l1weight_g.mean()) * UNPENALIZED

    inactive_ranks = inactive_results.shape[0] - rankdata(inactive_results)
    inactive_ranks[inactive_results <= 1 + tol] = -1

    return active_results, inactive_ranks




import numpy as np
import regreg.api as rr
import itertools
from numpy import testing as npt
from numpy.testing import dec

from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test()
@dec.setastest(True)
def test_affine_linear_offset_l1norm():

    """
    Test linear, affine and offset with the l1norm atom
    """
    
    n = 1000
    p = 10
    
    X = np.random.standard_normal((n,p))
    Y = 10*np.random.standard_normal(n)
    
    coefs = []
    
    loss = rr.quadratic_loss.affine(X,-Y, coef=0.5)
    sparsity = rr.l1norm(p, lagrange=5.)
    problem = rr.container(loss, sparsity)
    solver = rr.FISTA(problem)
    solver.fit(debug=False, tol=1e-10)
    coefs.append(1.*solver.composite.coefs)
    
    loss = rr.quadratic_loss.affine(X,-Y, coef=0.5)
    sparsity = rr.l1norm.linear(np.eye(p), lagrange=5.)
    problem = rr.container(loss, sparsity)
    solver = rr.FISTA(problem)
    solver.fit(debug=False, tol=1e-10)
    coefs.append(1.*solver.composite.coefs)
    
    loss = rr.quadratic_loss.affine(X,-Y, coef=0.5)
    sparsity = rr.l1norm.affine(np.eye(p),np.zeros(p), lagrange=5.)
    problem = rr.container(loss, sparsity)
    solver = rr.FISTA(problem)
    solver.fit(debug=False, tol=1e-10)
    coefs.append(1.*solver.composite.coefs)
    
    loss = rr.quadratic_loss.affine(X,-Y, coef=0.5)
    sparsity = rr.l1norm.linear(np.eye(p), lagrange=5., offset=np.zeros(p))
    problem = rr.container(loss, sparsity)
    solver = rr.FISTA(problem)
    solver.fit(debug=False, tol=1e-10)
    coefs.append(1.*solver.composite.coefs)
    
    loss = rr.quadratic_loss.affine(X,-Y, coef=0.5)
    sparsity = rr.l1norm.shift(np.zeros(p), lagrange=5.)
    problem = rr.container(loss, sparsity)
    solver = rr.FISTA(problem)
    solver.fit(debug=False, tol=1e-10)
    coefs.append(1.*solver.composite.coefs)

    for i,j in itertools.combinations(range(len(coefs)), 2):
        npt.assert_almost_equal(coefs[i], coefs[j])




"""
This module implements the notion of a separable support function / constraint
regularizer or penalty.

The penalty is specified by a primal shape, a sequence of atoms and
a sequence of slicing objects.
"""

import numpy as np

from ..affine import selector
from ..atoms import atom, affine_atom
from ..problems.simple import simple_problem
from ..atoms.cones import zero
from .composite import smooth_conjugate

def has_overlap(shape, groups):
    """
    Determine whether the groups, viewed as slices of an array
    with given shape, have any overlap.

    Parameters
    ----------
    shape : tuple
        A tuple of integers representing a shape for an array.
    groups : sequence
        A sequence of objects that can be viewed as slices of
        an ndarray with shape==shape.

    Returns
    -------
    res : boolean
        True if the slices overlap, else False.

    Examples
    --------

    >>> from regreg.problems.separable import has_overlap
    >>> has_overlap((4,5), [slice(2,3), slice(4,5)])
    False
    >>> has_overlap((4,5), [slice(2,3), [Ellipsis, slice(4,5)]])
    True
    """
    indices = np.arange(np.product(shape)).reshape(shape)
    subsets = []
    for group in groups:
        subsets.append(set(list(indices[group].reshape(-1))))
    subsets = tuple(subsets)
    for i in range(len(subsets)):
        for j in range(i):
            subset1 = subsets[i]
            subset2 = subsets[j]
            if subset1 != subset2 and subset1.intersection(subset2):
                    return True
    return False

class separable(atom):

    def __init__(self,
                 shape,
                 atoms,
                 groups,
                 test_for_overlap=False,
                 shapes=[],
                 initial=None):
        if test_for_overlap and has_overlap(shape, groups):
            raise ValueError('groups are not separable')
        self.shape = shape
        self.groups = groups
        self.atoms = atoms
        if shapes == []:
            shapes = [None] * len(atoms)
        self.shapes = shapes
        test = np.empty(self.shape)
        self.assign_shapes = [test[group].shape for group in groups]
        self.zero_atom = zero(shape)

        if initial is None:
            self.coefs = np.zeros(shape)

    def latexify(self, var=None, idx=''):
        # BUG idx is unused
        strs = []
        for i, atom in enumerate(self.atoms):
            template_dict = atom.objective_vars.copy()
            template_dict['linear'] = atom.objective_vars['linear']
            if var is not None:
                template_dict['var'] = "%s[g%d]" % (var, i)
            else:
                template_dict['var'] = "%s[g%d]" % (template_dict['var'], i)
            template_dict['idx'] = idx
            if isinstance(atom, affine_atom):
                strs.append(atom.latexify(var='%(linear)s_{%(idx)s}%(var)s' % template_dict, idx=i))
            else:
                strs.append(atom.latexify(var='%(var)s' % template_dict, idx=i))
        return ' + '.join(strs)

    def seminorm(self, x, lagrange=None, check_feasibility=False):
        value = 0.
        for atom, group, shape in zip(self.atoms,
                                      self.groups,
                                      self.shapes):
            x_group = x[group]
            if shape is not None:
                x_group.shape = shape
            value += atom.seminorm(x_group,
                                   lagrange=lagrange,
                                   check_feasibility=check_feasibility)
        return value

    def constraint(self, x, bound=None):
        value = 0.
        for atom, group, shape in zip(self.atoms,
                                      self.groups,
                                      self.shapes):
            x_group = x[group]
            if shape is not None:
                x_group.shape = shape
            value += atom.constraint(x_group, bound=bound)
        return value

    def nonsmooth_objective(self, x, check_feasibility=False):
        value = 0
        for atom, group, shape in zip(self.atoms,
                                      self.groups,
                                      self.shapes):
            x_group = x[group]
            if shape is not None:
                x_group.shape = shape
            value += atom.nonsmooth_objective(x_group,
                                              check_feasibility=check_feasibility)
        return value

    def proximal(self, proxq, prox_control=None):
        # This allows separable to not have every
        # coefficient penalized in a natural way
        # because this instantiates it at the prox map of the zero penalty
        v = self.zero_atom.proximal(proxq)
        for atom, group, shape, assign_shape in zip(self.atoms,
                                                    self.groups,
                                                    self.shapes,
                                                    self.assign_shapes):
            proxq_group = proxq[group]
            if shape is not None:
                proxq_group.linear_term.shape = shape
            v[group] = atom.proximal(proxq_group, prox_control).reshape(assign_shape)
        return v

    @property
    def conjugate(self):
        if self.quadratic.coef == 0:
            conjugate_atom = separable(self.shape,
                                       [atom.conjugate 
                                        for atom in self.atoms],
                                       self.groups,
                                       shapes=self.shapes)
        else:
            conjugate_atom = smooth_conjugate(self)
        self._conjugate = conjugate_atom
        self._conjugate._conjugate = self
        return self._conjugate

    def __repr__(self):
        return "separable(%s, %s, %s, %s)" % (repr(self.shape),
                                              repr(self.atoms),
                                              repr(self.groups),
                                              repr(self.shapes))

    @property
    def selectors(self):
        return [selector(group, self.shape, reshape)
                for group, shape in zip(self.groups, self.shapes)]

class separable_problem(simple_problem):
    
    def __init__(self,
                 smooth_atom,
                 shape,
                 atoms,
                 groups,
                 test_for_overlap=False,
                 shapes=[]):

        nonsmooth_atom = separable(shape,
                                   atoms,
                                   groups,
                                   test_for_overlap,
                                   shapes=shapes)
        simple_problem.__init__(self, smooth_atom, nonsmooth_atom)
        
    @staticmethod
    def singleton(atom, smooth_f):
        return separable_problem(smooth_f,
                                 atom.shape,
                                 [atom],
                                 [slice(None)],
                                 False)

    @staticmethod
    def fromatom(separable_atom, smooth_f):
        return separable_problem(smooth_f, separable_atom.shape, 
                                 separable_atom.atoms,
                                 separable_atom.groups,
                                 False,
                                 shapes=separable_atom.shapes)
    @property
    def selectors(self):
        return self.proximal_atom.selectors



import numpy as np
from scipy import sparse
from warnings import warn

from ..algorithms import FISTA
from ..problems.composite import (composite, nonsmooth as nonsmooth_composite,
                        smooth as smooth_composite)
from ..affine import (vstack as afvstack, identity as afidentity, power_L,
                     selector as afselector,
                     scalar_multiply, adjoint)
from ..problems.separable import separable
from ..smooth import smooth_atom, affine_smooth
from ..atoms import affine_atom as nonsmooth_affine_atom
from ..atoms.cones import zero_constraint, zero as zero_nonsmooth, affine_cone
from ..identity_quadratic import identity_quadratic

class dual_problem(composite):
    r"""
    A class for specifying a problem of the form

    .. math::

       \text{minimize}_{x} f(x) + g(Dx)

    which will be solved by a dual problem

    .. math::

       \text{minimize}_{u} f^*(-D^Tu) + g^*(u)

    while the primal variable is stored in the computation of the gradient of
    :math:`f^*`.
    """

    def __init__(self, f_conjugate, transform, atom):
        self.offset = None
        self.f_conjugate = f_conjugate
        if not isinstance(self.f_conjugate, smooth_composite):
            warn('the conjugate of f should be a smooth_composite to solve with generalized gradient')

        self.transform = transform
        self.atom = atom

        # the dual problem has f^*(-D^Tu) as objective
        self.affine_fc = affine_smooth(self.f_conjugate, scalar_multiply(adjoint(self.transform), -1))
        self.coefs = np.zeros(self.affine_fc.shape)

    # the quadratic is delegated to the individual parts
    @property
    def quadratic(self):
        return identity_quadratic(0,0,0,0)

    @staticmethod
    def fromprimal(f, *g):
        transform, separable_dual_atom = stacked_dual(f.shape, *g)
        return dual_problem(f.conjugate, transform, separable_dual_atom)

    def smooth_objective(self, x, mode='both', check_feasibility=False):
        """
        The smooth_objective DOES NOT INCLUDE the identity
        quadratic of all the smooth atoms.
        """
        v = self.affine_fc.smooth_objective(x, mode=mode, check_feasibility=check_feasibility)
        # retain a reference
        if mode in ['both', 'grad']:
            self.primal = self.affine_fc.grad
        return v

    def nonsmooth_objective(self, x, check_feasibility=False):
        out = self.atom.nonsmooth_objective(x, 
                                            check_feasibility=check_feasibility)
        return out + self.affine_fc.nonsmooth_objective(x, 
                                                        check_feasibility=check_feasibility)

    def proximal(self, proxq, prox_control=None):
        """
        The proximal function for the dual problem
        """
        return self.atom.proximal(proxq, prox_control=prox_control)

    def solve(self, quadratic=None, return_optimum=False, **fit_args):
        solver = FISTA(self)
        solver.fit(**fit_args)
        if return_optimum:
            return self.objective(self.primal), self.primal
        return self.primal

    def latexify(self, var=None):
        template_dict = self.objective_vars.copy()
        template_dict['prox'] =  self.atom.latexify(var=var,idx='2')
        template_dict['smooth'] = self.affine_fc.latexify(var=var,idx='1')
        template_dict
        result = r'''
        \begin{aligned}
        \text{minimize}_{%(var)s} & f(%(var)s) + g(%(var)s) \\
        f(%(var)s) &= %(smooth)s \\
        g(%(var)s) &= %(prox)s \\
        \end{aligned}
        ''' % template_dict
        result = '\n'.join([s.strip() for s in result.split('\n')])
        return result


def stacked_dual(shape, *primary_atoms):
    r'''
    Computes a dual of

    .. math::

       \sum_i g_i(D_i\beta)

    under the substitutions :math:`v_i=D_i\beta`.

    That is, it returns the following dual function after minimizing
    over :math:`(v_i,\beta_i)`:

    .. math::

       -\sum_i g_i^*(u_i)

    as well as the transform :math:`D \mapsto \mathbb{R}^p \prod_i
    \mathbb{R}^{m_i}` where :math:`p` is the primal shape and :math:`m_i` are
    the corresponding dual shapes.

    Parameters
    ----------
    primary_atoms : [atoms]
        Objects that have dual attributes, which is a pair
        (ltransform, conjugate).
    '''
    if len(primary_atoms) == 0:
        primary_atoms = [zero_nonsmooth(shape)]

    duals = [atom.dual for atom in primary_atoms]
    transforms = [d[0] for d in duals]
    dual_atoms = [d[1] for d in duals]

    if len(transforms) > 1:
        transform = afvstack(transforms)
        separable_atom = separable(transform.output_shape, dual_atoms,
                                   transform.output_slices)
        _dual = transform, separable_atom
    else:
        _dual = (transforms[0], dual_atoms[0])
    return _dual




import numpy as np

from .. import lasso, warm_start
from ...tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_gaussian(n=200,p=50):
    '''
    run a basic path algorithm

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    lasso1 = lasso.gaussian(X, Y, np.ones(X.shape[1]))
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        lagrange_proportion=0.2,
                                                        nstep=5) # initialized at "null" model
    sol1 = warm_start(lasso1, 
                      lagrange_sequence, 
                      (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                      inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_huber(n=200,p=50):
    '''
    run a basic path algorithm

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    lasso1 = lasso.huber(X, Y, 0.1, np.ones(X.shape[1]))
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        lagrange_proportion=0.2,
                                                        nstep=5) # initialized at "null" model
    sol1 = warm_start(lasso1, 
                      lagrange_sequence, 
                      (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                      inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_logistic(n=200,p=50):
    '''
    run a basic path algorithm

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.binomial(1, 0.5, size=(n,))

    lasso1 = lasso.logistic(X, Y, np.ones(X.shape[1]))
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        lagrange_proportion=0.2,
                                                        nstep=5) # initialized at "null" model
    sol1 = warm_start(lasso1, 
                      lagrange_sequence, 
                      (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                      inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_huber_svm(n=200,p=50):
    '''
    run a basic path algorithm

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.binomial(1, 0.5, size=(n,))

    lasso1 = lasso.huber_svm(X, Y, 0.1, np.ones(X.shape[1]))
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        lagrange_proportion=0.2,
                                                        nstep=5) # initialized at "null" model
    sol1 = warm_start(lasso1, 
                      lagrange_sequence, 
                      (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                      inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_poisson(n=200,p=50):
    '''
    run a basic path algorithm

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.poisson(5, size=(n,))

    lasso1 = lasso.poisson(X, Y, np.ones(X.shape[1]))
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        lagrange_proportion=0.2,
                                                        nstep=5) # initialized at "null" model
    sol1 = warm_start(lasso1, 
                      lagrange_sequence, 
                      (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                      inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_cox(n=200,p=50):
    '''
    run a basic path algorithm

    '''
    X = np.random.standard_normal((n,p))
    T = np.random.exponential(5, size=(n,))
    C = np.random.binomial(1, 0.5, size=(n,))
    lasso1 = lasso.cox(X, T, C, np.ones(X.shape[1]))
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        lagrange_proportion=0.2,
                                                        nstep=5) # initialized at "null" model
    sol1 = warm_start(lasso1, 
                      lagrange_sequence, 
                      (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                      inner_tol=1.e-5)
    beta1 = sol1['beta']



from copy import copy
from itertools import product

import numpy as np
import nose.tools as nt

import regreg.api as rr
from regreg.atoms.tests.test_seminorms import all_close
from regreg.tests.decorators import set_seed_for_test


@set_seed_for_test()
def test_class():
    '''
    runs several class methods on generic instance
    '''

    n, p = 100, 20
    X = np.random.standard_normal((n, p))
    Y = np.random.standard_normal(n)
    loss = rr.squared_error(X, Y)
    pen = rr.l1norm(p, lagrange=1.)
    problem = rr.simple_problem(loss, pen)

    problem.latexify()

    for debug, coef_stop, max_its in product([True, False], [True, False], [5, 100]):
        rr.gengrad(problem, rr.power_L(X)**2, max_its=max_its, debug=debug, coef_stop=coef_stop)

@set_seed_for_test()
def test_gengrad():
    Z = np.random.standard_normal(100) * 4
    p = rr.l1norm(100, lagrange=0.13)
    L = 0.14

    loss = rr.quadratic_loss.shift(Z, coef=L)
    problem = rr.simple_problem(loss, p)
    solver = rr.FISTA(problem)
    solver.fit(tol=1.0e-10, debug=True)

    simple_coef = solver.composite.coefs
    prox_coef = p.proximal(rr.identity_quadratic(L, Z, 0, 0))

    p2 = rr.l1norm(100, lagrange=0.13)
    p2 = copy(p)
    p2.quadratic = rr.identity_quadratic(L, Z, 0, 0)
    problem = rr.simple_problem.nonsmooth(p2)
    solver = rr.FISTA(problem)
    solver.fit(tol=1.0e-14, debug=True)
    simple_nonsmooth_coef = solver.composite.coefs

    p = rr.l1norm(100, lagrange=0.13)
    p.quadratic = rr.identity_quadratic(L, Z, 0, 0)
    problem = rr.simple_problem.nonsmooth(p)
    simple_nonsmooth_gengrad = rr.gengrad(problem, L, tol=1.0e-10)

    p = rr.l1norm(100, lagrange=0.13)
    problem = rr.separable_problem.singleton(p, loss)
    solver = rr.FISTA(problem)
    solver.fit(tol=1.0e-10)
    separable_coef = solver.composite.coefs

    loss2 = rr.quadratic_loss.shift(Z, coef=0.6*L)
    loss2.quadratic = rr.identity_quadratic(0.4*L, Z, 0, 0)
    p.coefs *= 0
    problem2 = rr.simple_problem(loss2, p)
    loss2_coefs = problem2.solve(coef_stop=True)
    solver2 = rr.FISTA(problem2)
    solver2.fit(tol=1.0e-10, debug=True, coef_stop=True)

    yield all_close, prox_coef, simple_nonsmooth_gengrad, 'prox to nonsmooth gengrad', None
    yield all_close, prox_coef, separable_coef, 'prox to separable', None
    yield all_close, prox_coef, simple_nonsmooth_coef, 'prox to simple_nonsmooth', None
    yield all_close, prox_coef, simple_coef, 'prox to simple', None
    yield all_close, prox_coef, loss2_coefs, 'simple where loss has quadratic 1', None
    yield all_close, prox_coef, solver2.composite.coefs, 'simple where loss has quadratic 2', None

@set_seed_for_test()
def test_gengrad_blocknorms():
    Z = np.random.standard_normal((10,10)) * 4
    p = rr.l1_l2((10,10), lagrange=0.13)
    dual = p.conjugate
    L = 0.23

    loss = rr.quadratic_loss.shift(Z, coef=L)
    problem = rr.simple_problem(loss, p)
    solver = rr.FISTA(problem)
    solver.fit(tol=1.0e-10, debug=True)
    simple_coef = solver.composite.coefs

    q = rr.identity_quadratic(L, Z, 0, 0)
    prox_coef = p.proximal(q)

    p2 = copy(p)
    p2.quadratic = rr.identity_quadratic(L, Z, 0, 0)
    problem = rr.simple_problem.nonsmooth(p2)
    solver = rr.FISTA(problem)
    solver.fit(tol=1.0e-14, debug=True)
    simple_nonsmooth_coef = solver.composite.coefs

    p = rr.l1_l2((10,10), lagrange=0.13)
    p.quadratic = rr.identity_quadratic(L, Z, 0, 0)
    problem = rr.simple_problem.nonsmooth(p)
    simple_nonsmooth_gengrad = rr.gengrad(problem, L, tol=1.0e-10)

    p = rr.l1_l2((10,10), lagrange=0.13)
    problem = rr.separable_problem.singleton(p, loss)
    solver = rr.FISTA(problem)
    solver.fit(tol=1.0e-10)
    separable_coef = solver.composite.coefs

    yield (all_close, prox_coef, simple_coef, 'prox to simple', None)
    yield (all_close, prox_coef, simple_nonsmooth_gengrad, 'prox to nonsmooth gengrad', None)
    yield (all_close, prox_coef, separable_coef, 'prox to separable', None)
    yield (all_close, prox_coef, simple_nonsmooth_coef, 'prox to simple_nonsmooth', None)





from __future__ import print_function, division, absolute_import

from itertools import chain

import numpy as np

import regreg.atoms.block_norms as B
import regreg.atoms.svd_norms as SVD
import regreg.api as rr
from regreg.tests.decorators import set_seed_for_test

import nose.tools as nt

from .test_seminorms import Solver, all_close, SolverFactory

class MatrixSolverFactory(SolverFactory):

    FISTA_choices = [True]
    L_choices = [0.3]
    coef_stop_choices = [False]
    shape = (5,4)

@set_seed_for_test(seed=20)
@np.testing.dec.slow
def test_proximal_maps():
    for klass in chain(B.conjugate_block_pairs.keys(),
                       SVD.conjugate_svd_pairs.keys()):
        if klass not in [B.block_max, B.block_sum]:
            factory = MatrixSolverFactory(klass, 'lagrange')
            for solver in factory:
                penalty = solver.atom
                dual = penalty.conjugate 
                Z = solver.prox_center
                L = solver.L

                yield all_close, penalty.lagrange_prox(Z, lipschitz=L), Z-dual.bound_prox(Z*L)/L, 'testing lagrange_prox and bound_prox starting from atom %s ' % klass, None

            # some arguments of the constructor

                yield nt.assert_raises, AttributeError, setattr, penalty, 'bound', 4.
                yield nt.assert_raises, AttributeError, setattr, dual, 'lagrange', 4.

                yield nt.assert_raises, AttributeError, setattr, penalty, 'bound', 4.
                yield nt.assert_raises, AttributeError, setattr, dual, 'lagrange', 4.

            for t in solver.all_tests():
                yield t

            factory = MatrixSolverFactory(klass, 'bound')
            for solver in factory:
                for t in solver.all_tests():
                    yield t





from __future__ import print_function, division, absolute_import

from copy import copy

import numpy as np

from ..identity_quadratic import identity_quadratic
from ..affine import (linear_transform, 
                      identity as identity_transform)
from ..objdoctemplates import objective_doc_templater
from ..doctemplates import (doc_template_user, 
                            doc_template_provider)
from ..problems.composite import smooth_conjugate
from ..atoms import atom, _work_out_conjugate, affine_atom
from .projl1_cython import projl1
from .piecewise_linear import find_solution_piecewise_linear_c

@objective_doc_templater()
class seminorm(atom):
    """
    An atom that can be in lagrange or bound form.
    """

    objective_template = r'\|%(var)s\|'
    objective_vars = {'var': r'\beta', 
                      'shape':'p', 
                      'linear':'D', 
                      'offset':r'\alpha',
                      'normklass':'l1norm',
                      'dualnormklass':'supnorm',
                      'initargs':'(30,)', # args need to construct penalty
                      }

    def __init__(self, shape, lagrange=None, bound=None,
                 offset=None, quadratic=None, initial=None):

        atom.__init__(self, shape, offset,
                      quadratic, initial)

        if not (bound is None or lagrange is None):
            raise ValueError('An atom must be either in Lagrange form or ' 
                             + 'bound form. Only one of the parameters '
                             + 'in the constructor can not be None.')
        if bound is None and lagrange is None:
            raise ValueError('Atom must be in lagrange or bound form, '
                             + 'as specified by the choice of one of'
                             + 'the keyword arguments.')
        if bound is not None and bound < 0:
            raise ValueError('Bound on the seminorm should be non-negative')
        if lagrange is not None and lagrange < 0:
            raise ValueError('Lagrange multiplier should be non-negative')

        if lagrange is not None:
            self._lagrange = lagrange
            self._bound = None
        if bound is not None:
            self._bound = bound
            self._lagrange = None
    
    def latexify(self, var=None, idx=''):
        r'''
        Return a LaTeX representation of an object.

        >>> import regreg.api as rr
        >>> penalty = rr.l1norm(10, lagrange=0.9)
        >>> penalty.latexify(var=r'\gamma') 
        '\\lambda_{} \\|\\gamma\\|_1'

        Parameters
        ----------

        var : `string`
            Argument of the functions

        idx : `string`
            Optional subscript index.

        Returns
        -------

        L : `string`
            A LaTeX representation of the atom.

        '''
        template_dict = self.objective_vars.copy()
        template_dict['idx'] = idx
        if var is not None:
            template_dict['var'] = var
        if self.offset is not None and np.any(self.offset != 0):
            template_dict['var'] = template_dict['var'] + (r' - %(offset)s_{%(idx)s}' % template_dict) 

        obj = self.objective_template % template_dict
        template_dict['obj'] = obj
        if self.lagrange is not None:
            obj = r'\lambda_{%(idx)s} %(obj)s' % template_dict
        else:
            obj = r'I^{\infty}(%(obj)s \leq \delta_{%(idx)s})' % template_dict

        if not self.quadratic.iszero:
            return ' + '.join([obj, self.quadratic.latexify(var=template_dict['var'], idx=idx)])
        return obj

    @doc_template_provider
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        r"""
        Return :math:`\lambda \cdot %(objective)s`, where
        :math:`\lambda` is `lagrange`. If `check_feasibility`
        is `True`, and seminorm is unbounded, will return `np.inf`
        if appropriate.

        The class seminorm's seminorm just returns the appropriate lagrange
        parameter for use by the subclasses.
        """
        return get_lagrange(self, lagrange)

    @doc_template_provider
    def constraint(self, arg, bound=None):
        r"""
        Verify :math:`%(objective)s \leq \delta`, where :math:`\delta`
        is `bound`.

        If the result is `True`, returns 0, else returns `np.inf`.

        The class seminorm's constraint just returns the appropriate bound
        parameter for use by the subclasses.
        """
        return get_bound(self, bound)

    def __eq__(self, other):
        if self.__class__ == other.__class__:
            if self.bound is not None:
                return self.bound == other.bound
            return self.lagrange == other.lagrange
        return False

    def __copy__(self):
        return self.__class__(copy(self.shape),
                              bound=copy(self.bound),
                              lagrange=copy(self.lagrange),
                              offset=copy(self.offset),
                              quadratic=copy(self.quadratic))

    def __repr__(self):
        if self.lagrange is not None:
            if self.quadratic.iszero:
                return "%s(%s, lagrange=%f, offset=%s)" % \
                    (self.__class__.__name__,
                     repr(self.shape), 
                     self.lagrange,
                     repr(self.offset))
            else:
                return "%s(%s, lagrange=%f, offset=%s, quadratic=%s)" % \
                    (self.__class__.__name__,
                     repr(self.shape), 
                     self.lagrange,
                     repr(self.offset),
                     repr(self.quadratic))

        else:
            if self.quadratic.iszero:
                return "%s(%s, bound=%f, offset=%s)" % \
                    (self.__class__.__name__,
                     repr(self.shape), 
                     self.bound,
                     repr(self.offset))

            else:
                return "%s(%s, bound=%f, offset=%s, quadratic=%s)" % \
                    (self.__class__.__name__,
                     repr(self.shape), 
                     self.bound,
                     repr(self.offset),
                     repr(self.quadratic))

    @doc_template_user
    @doc_template_provider
    def get_conjugate(self):
        """
        Return the conjugate of an given atom.

        >>> import regreg.api as rr
        >>> penalty = rr.%(normklass)s(%(initargs)s, lagrange=3.4)
        >>> penalty.get_conjugate() # doctest: +ELLIPSIS
        %(dualnormklass)s(..., bound=3.4...)

        """

        if self.quadratic.coef == 0:
            offset, outq = _work_out_conjugate(self.offset, self.quadratic)

            cls = conjugate_seminorm_pairs[self.__class__]
            conjugate_atom = cls(self.shape,  \
                       bound=self.lagrange, 
                       lagrange=self.bound,
                       quadratic=outq,
                       offset=offset)
        else:
            conjugate_atom = smooth_conjugate(self)
        self._conjugate = conjugate_atom
        self._conjugate._conjugate = self
        return self._conjugate
    conjugate = property(get_conjugate)

    @doc_template_user
    @doc_template_provider
    def get_lagrange(self):
        """
        Get method of the lagrange property.

        >>> import regreg.api as rr
        >>> penalty = rr.%(normklass)s(%(initargs)s, lagrange=3.4)
        >>> penalty.lagrange
        3.4

        """
        return self._lagrange

    @doc_template_user
    @doc_template_provider
    def set_lagrange(self, lagrange):
        """
        Set method of the lagrange property.

        >>> import regreg.api as rr
        >>> penalty = rr.%(normklass)s(%(initargs)s, lagrange=3.4)
        >>> penalty.lagrange
        3.4
        >>> penalty.lagrange = 2.3
        >>> penalty.lagrange
        2.3
        >>> constraint = rr.%(normklass)s(%(initargs)s, bound=3.4)
        >>> constraint.lagrange = 3.4 #doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
        ...
        AttributeError: atom is in bound mode

        """
        if self.bound is None:
            self._lagrange = lagrange
        else:
            raise AttributeError("atom is in bound mode")
    lagrange = property(get_lagrange, set_lagrange)

    @doc_template_user
    @doc_template_provider
    def get_bound(self):
        """
        Get method of the bound property.

        >>> import regreg.api as rr
        >>> constraint = rr.%(normklass)s(%(initargs)s, bound=2.3) 
        >>> constraint.bound
        2.3

        """
        return self._bound

    @doc_template_user
    @doc_template_provider
    def set_bound(self, bound):
        """
        Set method of the bound property.

        >>> import regreg.api as rr
        >>> constraint = rr.%(normklass)s(%(initargs)s, bound=3.4) 
        >>> constraint.bound
        3.4
        >>> constraint.bound = 2.3
        >>> constraint.bound
        2.3
        >>> penalty = rr.%(normklass)s(%(initargs)s, lagrange=2.3)
        >>> penalty.bound = 3.4 # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
        ...
        AttributeError: atom is in lagrange mode
        """
        if self.lagrange is None:
            self._bound = bound
        else:
            raise AttributeError("atom is in lagrange mode")
    bound = property(get_bound, set_bound)

    @doc_template_user
    @doc_template_provider
    def proximal(self, quadratic, prox_control=None):
        r"""
        The proximal operator. If the atom is in
        Lagrange mode, this has the form

        .. math::

           %(var)s^{\lambda}(\theta) = \text{argmin}_{%(var)s \in
           \mathbb{R}^{%(shape)s}} \frac{L}{2} \|\theta-%(var)s\|^2_2
           + \lambda h(%(var)s-\alpha) + \langle %(var)s, \eta \rangle

        where :math:`\alpha` is `self.offset`,
        :math:`\eta` is `quadratic.linear_term`, $\theta$ is `quadratic.center` and 

        .. math::

           h(%(var)s) = %(objective)s

        If the atom is in bound mode, then this has the form

        .. math::

           %(var)s^{\delta}(\theta) = \text{argmin}_{%(var)s \in \mathbb{R}^{%(shape)s}} \frac{L}{2}
           \|\theta-%(var)s\|^2_2 + \langle %(var)s, \eta \rangle \  \text{s.t.} \   
           h(%(var)s - \alpha) \leq \delta

        >>> import regreg.api as rr
        >>> penalty = rr.l1norm(4, lagrange=2)
        >>> Q = rr.identity_quadratic(1.5, [3, -4, -1, 1], 0, 0)
        >>> penalty.proximal(Q) # doctest: +ELLIPSIS
        array([ 1.6666..., -2.6666..., -0.        ,  0.        ])

        Parameters
        ----------

        quadratic : `regreg.identity_quadratic.identity_quadratic`

            A quadratic added to the atom before minimizing.

        prox_control : `[None, dict]`

            This argument is ignored for seminorms, but otherwise
            is passed to `regreg.algorithms.FISTA` if the atom
            needs to be solved iteratively.

        Returns
        -------

        Z : `np.ndarray(np.float)`
            The proximal map of the implied center of `quadratic`.

        """
        offset, totalq = (self.quadratic + quadratic).recenter(self.offset)

        if totalq.coef == 0:
            raise ValueError('lipschitz + quadratic coef must be positive')

        prox_arg = -totalq.linear_term / totalq.coef

        debug = False
        if debug:
            print('='*80)
            print('atom: ', self)
            print('quadratic: ', quadratic)
            print('proxarg: ', prox_arg)
            print('totalq: ', totalq)
            print('offset: ', offset)

        if self.bound is not None:
            eta = self.bound_prox(prox_arg, 
                                  bound=self.bound)
        else:
            eta = self.lagrange_prox(prox_arg, 
                                     lipschitz=totalq.coef, 
                                     lagrange=self.lagrange)

        if offset is None:
            return eta
        else:
            return eta + offset

    @doc_template_provider
    def lagrange_prox(self, arg, lipschitz=1, lagrange=None):
        r"""
        Return unique minimizer

        .. math::

           {%(var)s}^{\lambda}(\theta) =
           \text{argmin}_{%(var)s \in \mathbb{R}^{%(shape)s}} 
           \frac{L}{2}
           \|\theta-%(var)s\|^2_2 + \lambda %(objective)s 

        Above, :math:`\lambda` is the Lagrange parameter and :math:`L`
        is the Lipschitz parameter and $\theta$ is `arg`.

        If the argument `lagrange` is None and the atom is in lagrange mode,
        ``self.lagrange`` is used as the lagrange parameter, else an exception
        is raised.

        The class atom's lagrange_prox just returns the appropriate lagrange
        parameter for use by the subclasses.

        Parameters
        ----------

        arg : `np.ndarray(np.float)`
            Argument of the proximal map.

        lipschitz : `float`
            Coefficient in front of the quadratic.

        lagrange : `float` (optional)
            Lagrange factor in front of the seminorm. 
            Defaults to `self.lagrange`.

        Returns
        -------

        Z : `np.ndarray(np.float)`
            The proximal map of `arg`.

        """
        return get_lagrange(self, lagrange)

    @doc_template_provider
    def bound_prox(self, arg, bound=None):
        r"""
        Return unique minimizer

        .. math::

           {%(var)s}^{\delta}(\theta) =
           \text{argmin}_{%(var)s \in \mathbb{R}^{%(shape)s}} 
           \frac{1}{2}
           \|\theta-%(var)s\|^2_2  \ 
           \text{s.t.} \   %(objective)s \leq \delta

        where :math:`\delta` is the bound parameter and $\theta$ is `arg`. 

        If the argument `bound` is None and the atom is in bound mode,
        ``self.bound`` is used as the bound parameter, else an exception is
        raised.

        The class atom's bound_prox just returns the appropriate bound
        parameter for use by the subclasses.

        Parameters
        ----------

        arg : `np.ndarray(np.float)`
            Argument of the proximal map.

        bound : `float` (optional)
            Bound for the constraint on the seminorm.
            Defaults to `self.bound`.

        Returns
        -------

        Z : `np.ndarray(np.float)`
            The proximal map of `arg`.

        """
        return get_bound(self, bound)

    def nonsmooth_objective(self, arg, check_feasibility=False):
        """
        The nonsmooth objective function of the atom.
        Includes `self.quadratic.objective(arg)`.

        >>> import regreg.api as rr
        >>> penalty = rr.l1norm(4, lagrange=2)
        >>> penalty.nonsmooth_objective([3, 4, 5, 9])
        42.0
        >>> 2 * sum([3, 4, 5, 9])
        42


        Parameters
        ----------

        arg : `np.ndarray(np.float)`
            Argument of the seminorm.

        check_feasibility : `bool`
            If `True`, then return `np.inf` if appropriate.

        Returns
        -------

        value : `np.float`
            The seminorm of `arg`.

        """
        arg = np.asarray(arg)
        x_offset = self.apply_offset(arg)
        if self.bound is not None:
            if check_feasibility:
                value = self.constraint(x_offset)
            else:
                value = 0
        else:
            value = self.seminorm(x_offset, check_feasibility=check_feasibility)
        value += self.quadratic.objective(arg, 'func')
        return value

    @doc_template_user
    @doc_template_provider
    def get_dual(self):
        """
        Return the dual of an atom. This dual is formed by making introducing
        new variables $v=Ax$ where $A$ is `self.linear_transform`. 

        >>> import regreg.api as rr
        >>> penalty = rr.%(normklass)s(%(initargs)s, lagrange=2.3)
        >>> penalty # doctest: +ELLIPSIS
        %(normklass)s(..., lagrange=2.3...)
        >>> penalty.dual # doctest: +ELLIPSIS
        (<regreg.affine.identity object at 0x...>, %(dualnormklass)s(..., bound=2.3...))

        If there is a linear part to the penalty, the linear_transform may not be 
        identity. For example, the 1D fused LASSO penalty:

        >>> D = (np.identity(4) + np.diag(-np.ones(3),1))[:-1]
        >>> D
        array([[ 1., -1.,  0.,  0.],
               [ 0.,  1., -1.,  0.],
               [ 0.,  0.,  1., -1.]])
        >>> linear_atom = rr.l1norm.linear(D, lagrange=2.3)
        >>> linear_atom # doctest: +ELLIPSIS
        affine_atom(l1norm((3,), lagrange=2.3...), array([[ 1., -1.,  0.,  0.],
               [ 0.,  1., -1.,  0.],
               [ 0.,  0.,  1., -1.]]))
        >>> linear_atom.dual # doctest: +ELLIPSIS
        (<regreg.affine.linear_transform object at 0x...>, supnorm((3,), bound=2.3...))

        """
        return self.linear_transform, self.conjugate

    @classmethod
    def affine(cls, linear_operator, offset, lagrange=None, diag=False,
               bound=None, quadratic=None):
        """
        This is the same as the linear class method but with offset as a positional argument
        """
        if not isinstance(linear_operator, linear_transform):
            l = linear_transform(linear_operator, diag=diag)
        else:
            l = linear_operator
        # the minus signs below for offset is there until affine transforms SUBTRACT 
        # their offset until add. 
        # for atoms, the offset is really the "center"

        if offset is None:
            offset = 0
        new_atom = cls(l.output_shape, 
                       lagrange=lagrange, 
                       bound=bound,
                       offset=-offset,
                       quadratic=quadratic)
        return affine_atom(new_atom, l)

    @classmethod
    def linear(cls, linear_operator, lagrange=None, diag=False,
               bound=None, quadratic=None, offset=None):
        if not isinstance(linear_operator, linear_transform):
            l = linear_transform(linear_operator, diag=diag)
        else:
            l = linear_operator
        if offset is None:
            offset = 0
        new_atom = cls(l.output_shape, lagrange=lagrange, bound=bound,
                   quadratic=quadratic, offset=-offset)
        return affine_atom(new_atom, l)


    @classmethod
    def shift(cls, offset, lagrange=None, diag=False,
              bound=None, quadratic=None):
        new_atom = cls(offset.shape, lagrange=lagrange, bound=bound,
                       quadratic=quadratic, offset=-offset)
        return new_atom

    @staticmethod
    def check_subgradient(atom, prox_center):
        r"""
        For a given seminorm, verify the KKT condition for
        the problem for the proximal problem

        .. math::

            \text{minimize}_u \frac{1}{2} \|u-z\|^2_2 + h(z)

        where $z$ is the `prox_center` and $h$ is `atom`
        which may be in Lagrange or bound form.

        If the atom is in Lagrange form, this function should
        return two values equal to the seminorm of the 
        minimizer. If it is bound form it should return two values
        equal to the dual seminorm of the residual, i.e.
        the prox_center minus the minimizer.

        Parameters
        ----------

        atom : `seminorm`

        prox_center : np.ndarray(np.float)
             Center for the proximal map.

        Returns
        -------

        v1, v2 : float
             Two values that should be equal if the proximal map is correct.

        """
        atom = copy(atom)
        atom.quadratic = identity_quadratic(0,0,0,0)
        atom.offset = None
        q = identity_quadratic(1, prox_center, 0, 0)
        U = atom.proximal(q)
        dual = atom.conjugate
        if atom.bound is not None: # atom is bound mode
            if not atom.seminorm(U, lagrange=1) <= (1 + 1.e-5) * atom.bound:
                raise ValueError('expecting residual to have norm less than or equal to bound parameter')
            return ((prox_center - U) * U).sum() / atom.bound, dual.seminorm(prox_center-U, lagrange=1)
        else:
            if not dual.seminorm(prox_center - U, lagrange=1) <= (1 + 1.e-5) * atom.lagrange:
                raise ValueError('expecting residual to have norm less than equal to lagrange parameter: (%f, %f)' % (dual.seminorm(prox_center - U, lagrange=1), atom.lagrange))
            return ((prox_center - U) * U).sum() / atom.lagrange, atom.seminorm(U, lagrange=1)

@objective_doc_templater()
class l1norm(seminorm):

    """
    The l1 norm
    """
    prox_tol = 1.0e-10

    objective_template = r"""\|%(var)s\|_1"""
    objective_vars = seminorm.objective_vars.copy()
    objective_vars['normklass'] = 'l1norm'
    objective_vars['dualnormklass'] = 'supnorm'

    @doc_template_user
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        lagrange = seminorm.seminorm(self, arg, 
                                     check_feasibility=check_feasibility, 
                                     lagrange=lagrange)
        return lagrange * np.fabs(arg).sum()

    @doc_template_user
    def constraint(self, arg, bound=None):
        bound = seminorm.constraint(self, arg, bound=bound)
        inbox = np.fabs(arg).sum() <= bound * (1 + self.tol)
        if inbox:
            return 0
        else:
            return np.inf

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        lagrange = seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        return np.sign(arg) * np.maximum(np.fabs(arg)-lagrange/lipschitz, 0)

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        bound = seminorm.bound_prox(self, arg, bound)
        arg = np.asarray(arg, np.float)
        absarg = np.fabs(arg)
        cut = find_solution_piecewise_linear_c(bound, 0, absarg)
        if cut < np.inf:
            return np.sign(arg) * (absarg - cut) * (absarg > cut)
        return arg

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_conjugate(self):
        return seminorm.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return seminorm.dual(self)

@objective_doc_templater()
class supnorm(seminorm):

    r"""
    The :math:`\ell_{\infty}` norm
    """

    objective_template = r"""\|%(var)s\|_{\infty}"""
    objective_vars = seminorm.objective_vars.copy()
    objective_vars['normklass'] = 'supnorm'
    objective_vars['dualnormklass'] = 'l1norm'

    @doc_template_user
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        lagrange = seminorm.seminorm(self, arg, 
                                 check_feasibility=check_feasibility, 
                                 lagrange=lagrange)
        return lagrange * np.fabs(arg).max()

    @doc_template_user
    def constraint(self, arg, bound=None):
        bound = seminorm.constraint(self, arg, bound=bound)
        inbox = np.product(np.less_equal(np.fabs(arg), bound * (1+self.tol)))
        if inbox:
            return 0
        else:
            return np.inf

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        lagrange = seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        arg = np.asarray(arg, np.float)
        absarg = np.fabs(arg)
        cut = find_solution_piecewise_linear_c(lagrange / lipschitz, 0, absarg)
        if cut < np.inf:
            proj = np.sign(arg) * (absarg - cut) * (absarg > cut)
        else:
            proj = arg
        return arg - proj

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        bound = seminorm.bound_prox(self, arg, bound)
        return np.clip(arg, -bound, bound)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_conjugate(self):
        return seminorm.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return seminorm.dual(self)

@objective_doc_templater()
class l2norm(seminorm):

    """
    The l2 norm
    """

    objective_template = r"""\|%(var)s\|_2"""
    objective_vars = seminorm.objective_vars.copy()
    objective_vars['normklass'] = 'l2norm'
    objective_vars['dualnormklass'] = 'l2norm'

    @doc_template_user
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        lagrange = seminorm.seminorm(self, arg, 
                                     check_feasibility=check_feasibility, 
                                     lagrange=lagrange)
        return lagrange * np.linalg.norm(arg)

    @doc_template_user
    def constraint(self, arg, bound=None):
        bound = seminorm.constraint(self, arg, bound=bound)
        inball = (np.linalg.norm(arg) <= bound * (1 + self.tol))
        if inball:
            return 0
        else:
            return np.inf

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        lagrange = seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        n = np.linalg.norm(arg)
        if n <= lagrange / lipschitz:
            proj = arg
        else:
            proj = (lagrange / (lipschitz * n)) * arg
        return arg - proj

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        bound = seminorm.bound_prox(self, arg, bound)
        n = np.linalg.norm(arg)
        if n <= bound:
            return arg
        else:
            return (bound / n) * arg

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_conjugate(self):
        return seminorm.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return seminorm.dual(self)

def positive_part_lagrange(shape, lagrange,
                           offset=None, quadratic=None, initial=None):
    r'''
    The positive_part atom in lagrange form can be represented
    by an l1norm atom with the addition of a linear term
    and half the lagrange parameter. This reflects the fact that
    :math:`[0,1]^p = [-1/2,1/2]^p + 1/2 \pmb{1}`.

    '''
    lin = np.ones(shape) * .5 * lagrange
    linq = identity_quadratic(0,0,lin,0)
    if quadratic is not None:
        linq = linq + quadratic
    return l1norm(shape, lagrange=0.5*lagrange,
                  offset=offset, quadratic=linq,
                  initial=initial)

@objective_doc_templater()
class positive_part(seminorm):

    """
    The positive_part seminorm (which is the support
    function of [0,l]^p).
    """

    objective_template = r"""\left(\sum_{i=1}^{%(shape)s} (%(var)s)_i^+\right)"""
    objective_vars = seminorm.objective_vars.copy()
    objective_vars['normklass'] = 'positive_part'
    objective_vars['dualnormklass'] = 'constrained_max'

    @doc_template_user
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        lagrange = seminorm.seminorm(self, arg, 
                                 check_feasibility=check_feasibility, 
                                 lagrange=lagrange)
        return lagrange * np.maximum(arg, 0).sum()

    @doc_template_user
    def constraint(self, arg, bound=None):
        bound = seminorm.constraint(self, arg, bound=bound)
        inside = np.maximum(arg, 0).sum() <= bound * (1 + self.tol)
        if inside:
            return 0
        else:
            return np.inf

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        lagrange = seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        arg = np.asarray(arg)
        v = arg.copy()
        pos = v > 0
        v = np.atleast_1d(v)
        v[pos] = np.maximum(v[pos] - lagrange/lipschitz, 0)
        return v.reshape(arg.shape)

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        bound = seminorm.bound_prox(self, arg, bound)
        arg = np.asarray(arg)
        v = arg.copy().astype(np.float)
        v = np.atleast_1d(v)
        pos = v > 0
        if np.any(pos):
            v[pos] = projl1(v[pos], bound)
        return v.reshape(arg.shape)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_conjugate(self):
        return seminorm.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return seminorm.dual(self)

@objective_doc_templater()
class constrained_max(seminorm):
    r"""
    The seminorm x.max() s.t. x geq 0.
    """

    objective_template = (r"""\left\|%(var)s\right\|_{\infty} + """
                          + r"""I^{\infty}\left(\min(%(var)s) \in [0,+\infty)\right) """)
    objective_vars = seminorm.objective_vars.copy()
    objective_vars['normklass'] = 'constrained_max'
    objective_vars['dualnormklass'] = 'positive_part'

    @doc_template_user
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        lagrange = seminorm.seminorm(self, arg, 
                                 check_feasibility=check_feasibility, 
                                 lagrange=lagrange)
        anyneg = np.any(arg < 0 + self.tol)
        v = lagrange * np.max(arg)
        if not anyneg or not check_feasibility:
            return v
        return np.inf

    @doc_template_user
    def constraint(self, arg, bound=None):
        bound = seminorm.constraint(self, arg, bound=bound)
        anyneg = np.any(arg < 0 + self.tol)
        inside = np.max(arg) <= bound * (1 + self.tol)
        if inside and not anyneg:
            return 0
        else:
            return np.inf

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        lagrange = seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        arg = np.asarray(arg)
        v = arg.copy().astype(np.float)
        v = np.atleast_1d(v)
        pos = v > 0
        if np.any(pos):
            v[pos] = projl1(v[pos], lagrange/lipschitz)
        return arg - v.reshape(arg.shape)

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        bound = seminorm.bound_prox(self, arg, bound)
        return np.clip(arg, 0, bound)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_conjugate(self):
        return seminorm.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return seminorm.dual(self)

@objective_doc_templater()
class constrained_positive_part(seminorm):

    r"""
    Support function of $[-\infty,1]^p$
    """

    objective_template = (r"""\|%(var)s\|_{1} + \sum_{i=1}^{%(shape)s} """
                          + r"""\delta_{[0,+\infty]}(%(var)s_i)""")
    objective_vars = seminorm.objective_vars.copy()
    objective_vars['normklass'] = 'constrained_positive_part'
    objective_vars['dualnormklass'] = 'max_positive_part'

    @doc_template_user
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        lagrange = seminorm.seminorm(self, arg, 
                                 check_feasibility=check_feasibility, 
                                 lagrange=lagrange)
        anyneg = np.any(arg < 0 + self.tol)
        v = np.maximum(arg, 0).sum()
        if not anyneg or not check_feasibility:
            return v * lagrange
        return np.inf

    @doc_template_user
    def constraint(self, arg, bound=None):
        bound = seminorm.constraint(self, arg, bound=bound)
        value = self.seminorm(arg, lagrange=1, check_feasibility=True)
        if value >= bound * (1 + self.tol):
            return np.inf
        return 0

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        lagrange = seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        arg = np.asarray(arg)
        v = np.zeros(arg.shape)
        v = np.atleast_1d(v)
        pos = arg > 0
        if np.any(pos):
            v[pos] = np.maximum(arg[pos] - lagrange/lipschitz, 0)
        return v.reshape(arg.shape)

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        bound = seminorm.bound_prox(self, arg, bound)
        arg = np.asarray(arg)
        v = np.zeros(arg.shape, np.float)
        v = np.atleast_1d(v)
        pos = arg > 0
        if np.any(pos):
            v[pos] = projl1(arg[pos], bound)
        return v.reshape(arg.shape)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_conjugate(self):
        return seminorm.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return seminorm.dual(self)

@objective_doc_templater()
class max_positive_part(seminorm):

    """
    support function of the standard simplex
    """

    objective_template = r"""\|%(var)s^+\|_{\infty}"""
    objective_vars = seminorm.objective_vars.copy()
    objective_vars['normklass'] = 'max_positive_part'
    objective_vars['dualnormklass'] = 'constrained_positive_part'

    @doc_template_user
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        lagrange = seminorm.seminorm(self, arg, 
                                 check_feasibility=check_feasibility, 
                                 lagrange=lagrange)
        return np.max(np.maximum(arg,0)) * lagrange

    @doc_template_user
    def constraint(self, arg, bound=None):
        bound = seminorm.constraint(self, arg, bound=bound)
        v = np.max(np.maximum(arg,0))
        if v >= bound * (1 + self.tol):
            return np.inf
        return 0

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        bound = seminorm.bound_prox(self, arg, bound)
        return np.clip(arg, -np.inf, bound)

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        lagrange = seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        arg = np.asarray(arg)
        v = np.zeros(arg.shape, np.float)
        v = np.atleast_1d(v)
        pos = arg > 0
        if np.any(pos):
            v[pos] = projl1(arg[pos], lagrange / lipschitz)
        return arg - v.reshape(arg.shape)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_conjugate(self):
        return seminorm.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return seminorm.dual(self)

conjugate_seminorm_pairs = {}
for n1, n2 in [(l1norm,supnorm),
               (l2norm,l2norm),
               (positive_part, constrained_max),
               (constrained_positive_part, max_positive_part)]:
    conjugate_seminorm_pairs[n1] = n2
    conjugate_seminorm_pairs[n2] = n1

nonpaired_atoms = [positive_part_lagrange]

def get_lagrange(atom, lagrange=None):
    """
    Return appropriate `lagrange` parameter.
    """
    if lagrange is None:
        lagrange = atom.lagrange
    if lagrange is None:
        raise ValueError('either atom must be in Lagrange '
                         + 'mode or a keyword "lagrange" '
                         + 'argument must be supplied')
    return lagrange

def get_bound(atom, bound=None):
    """
    Return appropriate `bound` parameter.
    """
    if bound is None:
        bound = atom.bound
    if bound is None:
        raise ValueError('either atom must be in bound '
                         + 'mode or a keyword "bound" '
                         + 'argument must be supplied')
    return bound



from __future__ import print_function, division, absolute_import

from copy import copy
import nose.tools as nt

import numpy as np

import regreg.api as rr
from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_lasso_separable():
    """
    This test verifies that the specification of a separable
    penalty yields the same results as having two linear_atoms
    with selector matrices. The penalty here is a lasso, i.e. l1
    penalty.
    """

    X = np.random.standard_normal((100,20))
    Y = np.random.standard_normal((100,)) + np.dot(X, np.random.standard_normal(20))

    penalty1 = rr.l1norm(10, lagrange=1.2)
    penalty2 = rr.l1norm(10, lagrange=1.2)
    penalty = rr.separable((20,),
                           [penalty1, penalty2],
                           [slice(0,10), slice(10,20)],
                           test_for_overlap=True)

    # ensure code is tested

    print(penalty1.latexify())

    print(penalty.latexify())
    print(penalty.conjugate)
    print(penalty.dual)
    print(penalty.seminorm(np.ones(penalty.shape)))
    print(penalty.constraint(np.ones(penalty.shape), bound=2.))

    pencopy = copy(penalty)
    pencopy.set_quadratic(rr.identity_quadratic(1,0,0,0))
    pencopy.conjugate

    # solve using separable
    
    loss = rr.quadratic_loss.affine(X, -Y, coef=0.5)
    problem = rr.separable_problem.fromatom(penalty, loss)
    solver = rr.FISTA(problem)
    solver.fit(min_its=200, tol=1.0e-12)
    coefs = solver.composite.coefs

    # solve using the usual composite

    penalty_all = rr.l1norm(20, lagrange=1.2)
    problem_all = rr.container(loss, penalty_all)
    solver_all = rr.FISTA(problem_all)
    solver_all.fit(min_its=100, tol=1.0e-12)

    coefs_all = solver_all.composite.coefs

    # solve using the selectors

    penalty_s = [rr.linear_atom(p, rr.selector(g, (20,))) for p, g in
                 zip(penalty.atoms, penalty.groups)]
    problem_s = rr.container(loss, *penalty_s)
    solver_s = rr.FISTA(problem_s)
    solver_s.fit(min_its=500, tol=1.0e-12)
    coefs_s = solver_s.composite.coefs

    np.testing.assert_almost_equal(coefs, coefs_all)
    np.testing.assert_almost_equal(coefs, coefs_s)


    
@set_seed_for_test()
def test_group_lasso_separable():
    """
    This test verifies that the specification of a separable
    penalty yields the same results as having two linear_atoms
    with selector matrices. The penalty here is a group_lasso, i.e. l2
    penalty.
    """

    X = np.random.standard_normal((100,20))
    Y = np.random.standard_normal((100,)) + np.dot(X, np.random.standard_normal(20))

    penalty1 = rr.l2norm(10, lagrange=.2)
    penalty2 = rr.l2norm(10, lagrange=.2)
    penalty = rr.separable((20,), [penalty1, penalty2], [slice(0,10), slice(10,20)])

    # solve using separable
    
    loss = rr.quadratic_loss.affine(X, -Y, coef=0.5)
    problem = rr.separable_problem.fromatom(penalty, loss)
    solver = rr.FISTA(problem)
    solver.fit(min_its=200, tol=1.0e-12)
    coefs = solver.composite.coefs

    # solve using the selectors

    penalty_s = [rr.linear_atom(p, rr.selector(g, (20,))) for p, g in
                 zip(penalty.atoms, penalty.groups)]
    problem_s = rr.container(loss, *penalty_s)
    solver_s = rr.FISTA(problem_s)
    solver_s.fit(min_its=200, tol=1.0e-12)
    coefs_s = solver_s.composite.coefs

    np.testing.assert_almost_equal(coefs, coefs_s)


@set_seed_for_test()
def test_nonnegative_positive_part(debug=False):
    """
    This test verifies that using nonnegative constraint
    with a linear term, with some unpenalized terms yields the same result
    as using separable with constrained_positive_part and nonnegative
    """
    import numpy as np
    import regreg.api as rr
    import regreg.atoms as rra

    # N - number of data points
    # P - number of columns in design == number of betas
    N, P = 40, 30
    # an arbitrary positive offset for data and design
    offset = 2
    # data
    Y = np.random.normal(size=(N,)) + offset
    # design - with ones as last column
    X = np.ones((N,P))
    X[:,:-1] = np.random.normal(size=(N,P-1)) + offset
    # coef for loss
    coef = 0.5
    # lagrange for penalty
    lagrange = .1

    # Loss function (squared difference between fitted and actual data)
    loss = rr.quadratic_loss.affine(X, -Y, coef=coef)

    # Penalty using nonnegative, leave the last 5 unpenalized but
    # nonnegative
    weights = np.ones(P) * lagrange
    weights[-5:] = 0
    linq = rr.identity_quadratic(0,0,weights,0)
    penalty = rr.nonnegative(P, quadratic=linq)

    # Solution

    composite_form = rr.separable_problem.singleton(penalty, loss)
    solver = rr.FISTA(composite_form)
    solver.debug = debug
    solver.fit(tol=1.0e-12, min_its=200)
    coefs = solver.composite.coefs

    # using the separable penalty, only penalize the first
    # 25 coefficients with constrained_positive_part

    penalties_s = [rr.constrained_positive_part(25, lagrange=lagrange),
                   rr.nonnegative(5)]
    groups_s = [slice(0,25), slice(25,30)]
    penalty_s = rr.separable((P,), penalties_s,
                             groups_s)
    composite_form_s = rr.separable_problem.singleton(penalty_s, loss)
    solver_s = rr.FISTA(composite_form_s)
    solver_s.debug = debug
    solver_s.fit(tol=1.0e-12, min_its=200)
    coefs_s = solver_s.composite.coefs

    nt.assert_true(np.linalg.norm(coefs - coefs_s) / np.linalg.norm(coefs) < 1.0e-02)

@set_seed_for_test()
def test_different_dim():
    """
    This test checks that the reshape argument of separable
    works properly.
    """

    X = np.random.standard_normal((100,20))
    Y = (np.random.standard_normal((100,)) +
         np.dot(X, np.random.standard_normal(20)))

    penalty1 = rr.nuclear_norm((5, 2), lagrange=1.2)
    penalty2 = rr.l1norm(10, lagrange=1.2)
    penalty = rr.separable((20,),
                           [penalty1, penalty2],
                           [slice(0,10), slice(10,20)],
                           test_for_overlap=True,
                           shapes=[(5,2), None])

    # ensure code is tested

    print(penalty1.latexify())

    print(penalty.latexify())
    print(penalty.conjugate)
    print(penalty.dual)
    print(penalty.seminorm(np.ones(penalty.shape)))
    print(penalty.constraint(np.ones(penalty.shape), bound=2.))

    pencopy = copy(penalty)
    pencopy.set_quadratic(rr.identity_quadratic(1,0,0,0))
    pencopy.conjugate

    # solve using separable
    
    loss = rr.quadratic_loss.affine(X, -Y, coef=0.5)
    problem = rr.separable_problem.fromatom(penalty, loss)
    solver = rr.FISTA(problem)
    solver.fit(min_its=200, tol=1.0e-12)
    coefs = solver.composite.coefs



from __future__ import print_function, division, absolute_import

from warnings import warn
from copy import copy
import gc

import numpy as np
import numpy.linalg as npl

from scipy.stats import rankdata

from . import subsample_columns
from ..affine import power_L, normalize, astransform
from ..smooth import glm, affine_smooth, sum as smooth_sum
from ..smooth.quadratic import quadratic_loss
from ..problems.simple import simple_problem
from ..identity_quadratic import identity_quadratic as iq
from ..atoms.group_lasso import group_lasso
from . import grouped_path, default_lagrange_sequence

class group_lasso_path(grouped_path):

    BIG = 1e12 # lagrange parameter for finding null solution

    def __init__(self, 
                 saturated_loss,
                 X, 
                 groups,
                 weights={},
                 elastic_net_param=None,
                 alpha=1.  # elastic net mixing -- 1 is LASSO
                 ):

        self.saturated_loss = saturated_loss
        self.X = astransform(X)

        # the penalty parameters

        self.alpha = alpha
        self.penalty = group_lasso(groups, weights=weights, lagrange=1)
        self.group_shape = (len(np.unique(self.penalty.groups)),)
        self.shape = self.penalty.shape

        # elastic net part
        if elastic_net_param is None:
            elastic_net_param = np.ones(self.shape)
        self.elastic_net_param = elastic_net_param

        unpenalized_groups, unpenalized_idx = self.unpenalized
        self.solution = np.zeros(self.penalty.shape)

        self._unpenalized_vars = _candidate_bool(self.penalty.groups, 
                                                 unpenalized_groups)
        self._penalized_vars = np.ones(self.shape, np.bool)
        self._penalized_vars[self._unpenalized_vars] = 0

        if np.any(unpenalized_idx):
            (self.final_step, 
             null_grad, 
             null_soln,
             null_linpred,
             _) = self.solve_subproblem(self.solution,
                                        unpenalized_groups,
                                        self.BIG,
                                        tol=1.e-8)
            self.linear_predictor = null_linpred
            self.solution[self._unpenalized_vars] = null_soln
        else:
            self.linear_predictor = np.zeros(self.saturated_loss.shape)

        if np.any(self.elastic_net_param[self._unpenalized_vars]):
            warn('convention is that unpenalized parameters with have no Lagrange parameter in front '
                 'of their ridge term so that lambda_max is easily computed')

        self.grad_solution = (self.full_gradient(self.saturated_loss, 
                                                 self.linear_predictor) + self.enet_grad(self.solution, 
                                                                                         self._penalized_vars,
                                                                                         1))

    # method potentially overwritten in subclasses for penalty considerations

    def enet_loss(self,
                  lagrange,
                  candidate_groups=None):

        return _restricted_elastic_net(self.elastic_net_param, 
                                       self._penalized_vars,
                                       self.penalty.groups,
                                       lagrange,
                                       self.alpha,
                                       candidate_groups)

    def subsample(self,
                  case_idx):
        '''

        Create a new path, by subsampling
        cases of `self.saturated_loss`.

        Case weights are computed
        with `self.saturated_loss.subsample`.

        Parameters
        ----------

        case_idx : index
            An index-like object used 
            to specify which cases to include
            in the subsample.

        Returns
        -------

        subsample_path : path object
            A path object with a modified smooth part
            reflecting the subsampling.

        '''
        subsample_loss = self.saturated_loss.subsample(case_idx)
        return self.__class__(subsample_loss,
                              self.X,
                              self.penalty.groups,
                              weights=self.penalty.weights,
                              elastic_net_param=self.elastic_net_param,
                              alpha=self.alpha)

    def check_KKT(self,
                  grad_solution,
                  solution,
                  lagrange,
                  penalty=None):

        '''

        Check KKT conditions over
        the groups in the path.
        Returns boolean indicating
        which groups are failing the KKT conditions
        (these could be `active` groups or
        `inactive` groups).

        Parameters
        ----------

        grad_solution : ndarray
             Candidate for gradient of smooth loss at 
             Lagrange value `lagrange`.

        solution : ndarray
             Candidate for solution to problem 
             Lagrange value `lagrange`.

        lagrange : float
             Lagrange value for penalty

        penalty : object (optional)
             A group LASSO penalty. If None, defaults
             to `self.penalty`.

        '''

        if penalty is None:
            penalty = self.penalty

        active, inactive_ranks = _check_KKT(penalty,
                                            grad_solution, 
                                            solution, 
                                            lagrange)
        return active > 0, inactive_ranks

    def strong_set(self,
                   lagrange_cur,
                   lagrange_new,
                   grad_solution):

        _strong_bool = _strong_set(self.penalty,
                                   lagrange_cur,
                                   lagrange_new,
                                   grad_solution)
        _strong = np.array([self.penalty._sorted_groupids[i] for i in np.nonzero(_strong_bool)[0]])
        return (_strong,
                _strong_bool,
                np.nonzero(_candidate_bool(self.penalty.groups, _strong))[0])
                                             
    def solve_subproblem(self, 
                         solution,
                         candidate_groups, 
                         lagrange_new, 
                         **solve_args):
    
        # solve a problem with a candidate set

        sub_loss, sub_penalty, sub_X, candidate_bool = _restricted_problem(self.X, 
                                                                           self.saturated_loss, 
                                                                           self.alpha * lagrange_new, 
                                                                           self.penalty.groups,
                                                                           self.penalty.weights,
                                                                           candidate_groups,
                                                                           self.subsample_columns)
        if self.alpha < 1:
            sub_elastic_net = self.enet_loss(lagrange_new,
                                             candidate_groups)
            sub_loss = smooth_sum([sub_loss, sub_elastic_net])

        sub_problem = simple_problem(sub_loss, sub_penalty)
        sub_problem.coefs[:] = solution[candidate_bool] # warm start
        sub_soln = sub_problem.solve(**solve_args)
        sub_grad = sub_loss.smooth_objective(sub_soln, mode='grad') 
        sub_linear_pred = sub_X.dot(sub_soln)
        return sub_problem.final_step, sub_grad, sub_soln, sub_linear_pred, candidate_bool

    def enet_grad(self,
                  solution,
                  penalized, # boolean
                  lagrange_new,
                  subset=None):

        elastic_net_param = self.elastic_net_param
        if subset is not None:
            solution = solution[subset]
            penalized = penalized[subset]
            elastic_net_param = elastic_net_param[subset]
        G = (1 - self.alpha) * solution * elastic_net_param
        G[penalized] *= lagrange_new
        return G

    def updated_ever_active(self,
                            index_obj,
                            group_ids=True):
        if not hasattr(self, '_ever_active'):
            self._ever_active = np.zeros(self.group_shape, np.bool)
            self._sorted_groupids = np.array(self.penalty._sorted_groupids)
        _ever_active = self._ever_active.copy()
        _ever_active[index_obj] = True
        if group_ids:
            return list(self._sorted_groupids[_ever_active])
        else:
            return list(np.nonzero(_ever_active)[0])

    @property
    def unpenalized(self):
        """
        Unpenalized groups and variables.

        Returns
        -------

        groups : sequence
            Groups with weights equal to 0.

        variables : ndarray
            Boolean indicator that is True if no penalty on that variable.

        """
        if not hasattr(self, "_unpen_groups"):
            self._unpen_groups = []
            self._unpen_group_idx = np.zeros(self.group_shape, np.bool)
            for i, g in enumerate(self.penalty._sorted_groupids):
                unpen_group = self.penalty.weights[g] == 0
                self._unpen_group_idx[i] = unpen_group
                if unpen_group:
                    self._unpen_groups.append(g)
        return self._unpen_groups, self._unpen_group_idx

    def active_set(self, solution):
        return self.penalty.terms(solution) != 0

    def restricted_penalty(self, var_subset):
        if var_subset is not None:
            groups = self.penalty.groups[var_subset]
        else:
            groups = self.penalty.groups
        return group_lasso(groups,
                           weights=self.penalty.weights,
                           lagrange=1)

# Some common loss factories

def logistic(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return group_lasso_path(glm.logistic_loglike(Y.shape, Y), X, *args, **keyword_args)

def probit(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return group_lasso_path(glm.probit_loglike(Y.shape, Y), X, *args, **keyword_args)

def cloglog(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return group_lasso_path(glm.cloglog_loglike(Y.shape, Y), X, *args, **keyword_args)

def gaussian(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return group_lasso_path(glm.gaussian_loglike(Y.shape, Y), X, *args, **keyword_args)

def cox(X, T, S, *args, **keyword_args):
    T, S = np.asarray(T), np.asarray(S)
    return group_lasso_path(glm.cox_loglike(T.shape, T, S), X, *args, **keyword_args)

def poisson(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return group_lasso_path(glm.poisson_loglike(Y.shape, Y), X, *args, **keyword_args)

def huber(X, Y, smoothing_parameter, *args, **keyword_args):
    Y = np.asarray(Y)
    return group_lasso_path(glm.huber_loss(Y.shape, Y, smoothing_parameter), X, *args, **keyword_args)

def huber_svm(X, Y, smoothing_parameter, *args, **keyword_args):
    Y = np.asarray(Y)
    return group_lasso_path(glm.huber_svm(Y.shape, Y, smoothing_parameter), X, *args, **keyword_args)

# private functions

def _candidate_bool(groups, candidate_groups):

    candidate_bool = np.zeros(groups.shape, np.bool)
    for g in candidate_groups:
        group = groups == g
        candidate_bool += group

    return candidate_bool

def _strong_set(penalty,
                lagrange_cur,
                lagrange_new,
                gradient,
                slope_estimate=1):
    """
    Return a Boolean indicator for each group
    indicating whether in the strong set or not.
    """

    thresh = (slope_estimate + 1) * lagrange_new - slope_estimate * lagrange_cur
    dual = penalty.conjugate
    return np.asarray(dual.terms(gradient, check_feasibility=True)) > thresh

def _restricted_elastic_net(elastic_net_params, 
                            penalized,
                            groups,
                            lagrange, 
                            alpha,
                            candidate_groups):

    new_params = elastic_net_params * (1 - alpha)
    new_params[penalized] *= lagrange 
    if candidate_groups is not None:
        candidate_bool = _candidate_bool(groups, candidate_groups)
        new_params = new_params[candidate_bool]
    return quadratic_loss(new_params.shape,
                          new_params,
                          Qdiag=True)

def _restricted_problem(X, 
                        saturated_loss, 
                        alpha,
                        groups,
                        group_weights, 
                        candidate_groups,
                        subsample_columns):

    candidate_bool = _candidate_bool(groups, candidate_groups)
    X_candidate = subsample_columns(X, np.nonzero(candidate_bool)[0])
    restricted_penalty = group_lasso(groups[candidate_bool],
                                     weights=copy(group_weights), # should be OK to reuse weights dict
                                     lagrange=alpha)

    restricted_loss = affine_smooth(saturated_loss, X_candidate)
    return restricted_loss, restricted_penalty, X_candidate, candidate_bool

def _check_KKT(penalty,
               grad, 
               solution, 
               lagrange, 
               tol=1.e-2):

    """
    Check whether (grad, solution) satisfy
    KKT conditions at a given tolerance.
    """
    dual = penalty.conjugate
    terms = dual.terms(solution)

    ACTIVE = 1
    INACTIVE = 2
    UNPENALIZED = 3

    active_results = np.zeros(len(penalty._sorted_groupids), np.int)
    inactive_results = np.zeros(len(penalty._sorted_groupids), np.float)
    for i, g in enumerate(penalty._sorted_groupids):
        group = penalty.groups == g
        w = penalty.weights[g]
        active = (terms[i] != 0) * (w > 0)
        unpenalized = w == 0
        if active:
            active_results[i] = (np.linalg.norm(solution[group] * lagrange * w / np.linalg.norm(solution[group]) + grad[group]) > tol) * ACTIVE
        elif unpenalized:
            active_results[i] = (np.linalg.norm(grad[group]) > tol * np.mean(penalty._weight_array)) * UNPENALIZED
        else:
            inactive_results[i] = np.linalg.norm(grad[group]) / (lagrange * w) 
    # rank inactive groups

    inactive_ranks = inactive_results.shape[0]  - rankdata(inactive_results)
    inactive_ranks[inactive_results <= 1 + tol] = -1

    return active_results, inactive_ranks



from copy import copy

import nose.tools as nt
import numpy as np
from scipy.stats import norm as normal_dbn

from .. import glm

def test_logistic():

    for Y, T in [(np.random.binomial(1,0.5,size=(10,)),
                  np.ones(10)),
                 (np.random.binomial(1,0.5,size=(10,)),
                  None),
                 (np.random.binomial(3,0.5,size=(10,)),
                  3*np.ones(10))]:
        X = np.random.standard_normal((10,5))

        for case_weights in [None, np.ones(10)]:
            L = glm.glm.logistic(X, Y, trials=T, case_weights=case_weights)
            L.smooth_objective(np.zeros(L.shape), 'both')
            L.hessian(np.zeros(L.shape))

            sat_sub = L.saturated_loss.subsample(np.arange(5)) # check that subsample of saturated loss at least works
            sat_sub.smooth_objective(np.zeros(sat_sub.shape))

            # check that subsample is getting correct answer

            Xsub = X[np.arange(5)]
            Ysub = Y[np.arange(5)]
            if T is not None:
                Tsub = T[np.arange(5)]
                T_num = T
            else:
                Tsub = np.ones(5)
                T_num = np.ones(10)

            beta = np.ones(L.shape)

            if case_weights is not None:
                Lsub2 = glm.glm.logistic(Xsub, Ysub, trials=Tsub, case_weights=case_weights[np.arange(5)])
                Lsub3 = glm.glm.logistic(Xsub, Ysub, trials=Tsub, case_weights=case_weights[np.arange(5)])
                case_cp = case_weights.copy() * 0
                case_cp[np.arange(5)] = 1
                Lsub4 = glm.glm.logistic(X, Y, trials=T, case_weights=case_cp)
            else:
                Lsub2 = glm.glm.logistic(Xsub, Ysub, trials=Tsub)
                Lsub3 = glm.glm.logistic(Xsub, Ysub, trials=Tsub)

            Lsub3.coef *= 2.

            f2, g2 = Lsub2.smooth_objective(beta, 'both')
            f3, g3 = Lsub3.smooth_objective(beta, 'both')
            f4, g4 = Lsub2.smooth_objective(beta, 'both')

            np.testing.assert_allclose(f3, 2*f2)
            np.testing.assert_allclose(g3, 2*g2)

            np.testing.assert_allclose(f2, f4)
            np.testing.assert_allclose(g2, g4)

            Lcp = copy(L)
            prev_value = L.smooth_objective(np.zeros(L.shape), 'func')
            L_sub = L.subsample(np.arange(5))
            L_sub.coef *= 45
            new_value = L.smooth_objective(np.zeros(L.shape), 'func')            
            assert(prev_value == new_value)
            

            np.testing.assert_allclose(L_sub.gradient(beta),
                                       45 * Lsub2.gradient(beta))

            np.testing.assert_allclose(L.gradient(beta),
                                       X.T.dot(L.saturated_loss.mean_function(X.dot(beta)) * T_num - Y))

            np.testing.assert_allclose(L_sub.gradient(beta),
                                       45 * Xsub.T.dot(L_sub.saturated_loss.mean_function(Xsub.dot(beta)) * Tsub - Ysub))

            np.testing.assert_allclose(L_sub.gradient(beta),
                                       45 * Xsub.T.dot(Lsub2.saturated_loss.mean_function(Xsub.dot(beta)) * Tsub - Ysub))

            # other checks on gradient

            if T is None:
                np.testing.assert_allclose(L.gradient(np.zeros(L.shape)),
                                           X.T.dot(0.5 - Y))
                np.testing.assert_allclose(L.hessian(np.zeros(L.shape)),
                                           X.T.dot(X) / 4.)
            else:
                L.gradient(np.zeros(L.shape))
                L.hessian(np.zeros(L.shape))

            L.objective(np.zeros(L.shape))
            L.latexify()

            L.saturated_loss.data = (Y, T)
            L.saturated_loss.data

            L.data = (X, (Y, T))
            L.data

def test_probit():

    for Y, T in [(np.random.binomial(1,0.5,size=(10,)),
                  np.ones(10)),
                 (np.random.binomial(1,0.5,size=(10,)),
                  None),
                 (np.random.binomial(3,0.5,size=(10,)),
                  3*np.ones(10))]:
        X = np.random.standard_normal((10,5))

        for case_weights in [None, np.ones(10)]:
            L = glm.glm.probit(X, Y, trials=T, case_weights=case_weights)
            L.smooth_objective(np.zeros(L.shape), 'both')
            L.hessian(np.zeros(L.shape))

            sat_sub = L.saturated_loss.subsample(np.arange(5)) # check that subsample of saturated loss at least works
            sat_sub.smooth_objective(np.zeros(sat_sub.shape))

            # check that subsample is getting correct answer

            Xsub = X[np.arange(5)]
            Ysub = Y[np.arange(5)]
            if T is not None:
                Tsub = T[np.arange(5)]
                T_num = T
            else:
                Tsub = np.ones(5)
                T_num = np.ones(10)

            beta = np.ones(L.shape)

            if case_weights is not None:
                Lsub2 = glm.glm.probit(Xsub, Ysub, trials=Tsub, case_weights=case_weights[np.arange(5)])
                Lsub3 = glm.glm.probit(Xsub, Ysub, trials=Tsub, case_weights=case_weights[np.arange(5)])
                case_cp = case_weights.copy() * 0
                case_cp[np.arange(5)] = 1
                Lsub4 = glm.glm.probit(X, Y, trials=T, case_weights=case_cp)
            else:
                Lsub2 = glm.glm.probit(Xsub, Ysub, trials=Tsub)
                Lsub3 = glm.glm.probit(Xsub, Ysub, trials=Tsub)

            Lsub3.coef *= 2.

            f2, g2 = Lsub2.smooth_objective(beta, 'both')
            f3, g3 = Lsub3.smooth_objective(beta, 'both')
            f4, g4 = Lsub2.smooth_objective(beta, 'both')

            np.testing.assert_allclose(f3, 2*f2)
            np.testing.assert_allclose(g3, 2*g2)

            np.testing.assert_allclose(f2, f4)
            np.testing.assert_allclose(g2, g4)

            Lcp = copy(L)
            prev_value = L.smooth_objective(np.zeros(L.shape), 'func')
            L_sub = L.subsample(np.arange(5))
            L_sub.coef *= 45
            new_value = L.smooth_objective(np.zeros(L.shape), 'func')            
            assert(prev_value == new_value)
            

            np.testing.assert_allclose(L_sub.gradient(beta),
                                       45 * Lsub2.gradient(beta))

            linpred = X.dot(beta)
            np.testing.assert_allclose(L.gradient(beta),
                                       X.T.dot(-normal_dbn.pdf(linpred) * (Y / normal_dbn.cdf(linpred) - 
                                                                           (T_num - Y) / normal_dbn.sf(linpred))))

            linpred = Xsub.dot(beta)
            np.testing.assert_allclose(L_sub.gradient(beta),
                                       45 * Xsub.T.dot(-normal_dbn.pdf(linpred) * 
                                                        (Ysub / normal_dbn.cdf(linpred) - 
                                                         (Tsub - Ysub) / normal_dbn.sf(linpred))))

            # other checks on gradient

            if T is None:
                sat = L.saturated_loss
                np.testing.assert_allclose(sat.smooth_objective(np.zeros(sat.shape), 'grad'),
                                           (0.5 - Y) * normal_dbn.pdf(0) / 0.25)
                np.testing.assert_allclose(L.gradient(np.zeros(L.shape)),
                                           X.T.dot(0.5 - Y) * normal_dbn.pdf(0) / 0.25)
                np.testing.assert_allclose(L.hessian(np.zeros(L.shape)),
                                           X.T.dot(X) / 0.25 * normal_dbn.pdf(0)**2)
            else:
                L.gradient(np.zeros(L.shape))
                L.hessian(np.zeros(L.shape))

            L.objective(np.zeros(L.shape))
            L.latexify()

            L.saturated_loss.data = (Y, T)
            L.saturated_loss.data

            L.data = (X, (Y, T))
            L.data

def test_cloglog():

    for Y, T in [(np.random.binomial(1,0.5,size=(10,)),
                  np.ones(10)),
                 (np.random.binomial(1,0.5,size=(10,)),
                  None),
                 (np.random.binomial(3,0.5,size=(10,)),
                  3*np.ones(10))]:
        X = np.random.standard_normal((10,5))

        for case_weights in [None, np.ones(10)]:
            L = glm.glm.cloglog(X, Y, trials=T, case_weights=case_weights)
            L.smooth_objective(np.zeros(L.shape), 'both')
            L.hessian(np.zeros(L.shape))

            sat_sub = L.saturated_loss.subsample(np.arange(5)) # check that subsample of saturated loss at least works
            sat_sub.smooth_objective(np.zeros(sat_sub.shape))

            # check that subsample is getting correct answer

            Xsub = X[np.arange(5)]
            Ysub = Y[np.arange(5)]
            if T is not None:
                Tsub = T[np.arange(5)]
                T_num = T
            else:
                Tsub = np.ones(5)
                T_num = np.ones(10)

            beta = np.ones(L.shape)

            if case_weights is not None:
                Lsub2 = glm.glm.cloglog(Xsub, Ysub, trials=Tsub, case_weights=case_weights[np.arange(5)])
                Lsub3 = glm.glm.cloglog(Xsub, Ysub, trials=Tsub, case_weights=case_weights[np.arange(5)])
                case_cp = case_weights.copy() * 0
                case_cp[np.arange(5)] = 1
                Lsub4 = glm.glm.cloglog(X, Y, trials=T, case_weights=case_cp)
            else:
                Lsub2 = glm.glm.cloglog(Xsub, Ysub, trials=Tsub)
                Lsub3 = glm.glm.cloglog(Xsub, Ysub, trials=Tsub)

            Lsub3.coef *= 2.

            f2, g2 = Lsub2.smooth_objective(beta, 'both')
            f3, g3 = Lsub3.smooth_objective(beta, 'both')
            f4, g4 = Lsub2.smooth_objective(beta, 'both')

            np.testing.assert_allclose(f3, 2*f2)
            np.testing.assert_allclose(g3, 2*g2)

            np.testing.assert_allclose(f2, f4)
            np.testing.assert_allclose(g2, g4)

            Lcp = copy(L)
            prev_value = L.smooth_objective(np.zeros(L.shape), 'func')
            L_sub = L.subsample(np.arange(5))
            L_sub.coef *= 45
            new_value = L.smooth_objective(np.zeros(L.shape), 'func')            
            assert(prev_value == new_value)
            
            np.testing.assert_allclose(L_sub.gradient(beta),
                                       45 * Lsub2.gradient(beta))

            linpred = X.dot(0.1 * beta)
            cdf = 1 - np.exp(-np.exp(linpred))
            sf = np.exp(-np.exp(linpred))
            pdf = np.exp(linpred - np.exp(linpred))

            np.testing.assert_allclose(L.gradient(0.1 * beta),
                                       X.T.dot(-pdf * (Y / cdf - 
                                                       (T_num - Y) / sf)))

            linpred = Xsub.dot(0.1 * beta)
            cdf = 1 - np.exp(-np.exp(linpred))
            sf = np.exp(-np.exp(linpred))
            pdf = np.exp(linpred - np.exp(linpred))

            np.testing.assert_allclose(L_sub.gradient(0.1 * beta),
                                       45 * Xsub.T.dot(-pdf * 
                                                        (Ysub / cdf - 
                                                         (Tsub - Ysub) / sf)))

            # other checks on gradient

            if T is None:
                sat = L.saturated_loss
                pdf0 = np.exp(-1)
                cdf = (1 - np.exp(-1))
                sf = np.exp(-1)
                denom = cdf * sf
                np.testing.assert_allclose(sat.smooth_objective(np.zeros(sat.shape), 'grad'),
                                           (cdf - Y) * pdf0 / denom)
                np.testing.assert_allclose(L.gradient(np.zeros(L.shape)),
                                           X.T.dot(cdf - Y) * pdf0 / denom)
                np.testing.assert_allclose(L.hessian(np.zeros(L.shape)),
                                           X.T.dot(X) / denom * pdf0**2)
            else:
                L.gradient(np.zeros(L.shape))
                L.hessian(np.zeros(L.shape))

            L.objective(np.zeros(L.shape))
            L.latexify()

            L.saturated_loss.data = (Y, T)
            L.saturated_loss.data

            L.data = (X, (Y, T))
            L.data




import numpy as np
import regreg.api as rr
import nose.tools as nt
from ...tests.decorators import set_seed_for_test

@set_seed_for_test()
@np.testing.dec.skipif(True, msg='NESTA path not implemented correctly')
def test_nesta_path():

    def atom_factory(candidate_set):
        return rr.nonnegative.linear(constraint_matrix[:,candidate_set])

    Z2 = np.zeros((100,8))
    Z2[:,:3] = X[:,:3]
    Z2[:,3:6] = -X[:,:3]
    Z2[:,6:] = -X[:,3:]

    constraint_matrix = np.zeros((3,9))
    constraint_matrix[2,1:6] = 1
    constraint_matrix[0,6] = 1
    constraint_matrix[1,7] = 1
    constraint = rr.nonnegative.linear(constraint_matrix)

    lasso_constraint = rr.nesta_path.squared_error(Z2, Y, atom_factory, nstep=10)
    sol4 = lasso_constraint.main()



import numpy as np
try:
    from sklearn.model_selection import cross_validate
    from sklearn.model_selection import GridSearchCV
    from ..api import (multinomial,
                       multinomial_lagrange,
                       multinomial_classifier,
                       multinomial_classifier_lagrange)
    have_sklearn = True
except ImportError:
    have_sklearn = False

from ...api import l1_l2
from ...tests.decorators import set_seed_for_test

@np.testing.dec.skipif(True or not have_sklearn, msg='multinomial not working on its own yet')
@set_seed_for_test()
def test_multinomial():

    n, p, q = 100, 20, 3
    X = np.random.standard_normal((n, p))
    Z = np.random.standard_normal((n, q))
    A = np.argmax(Z, 1)
    Y = np.zeros_like(Z)
    for i in range(n):
        Y[i,A[i]] = 1
    response = Y

    atom = l1_l2
    atom_args = {'shape':(p, q), 'bound':3}

    multinomial_lasso = multinomial(atom,
                                    atom_args)

    multinomial_lasso.fit(X, response)
    print(cross_validate(multinomial_lasso, X, response, cv=10))

    # grid search
    params = {'atom_params':[{'shape':(p, q),
                              'bound': b} for b in [3, 4, 5]]}
    lasso_cv = GridSearchCV(multinomial_lasso, params, cv=3)
    lasso_cv.fit(X, response)

    multinomial_lasso_offset = multinomial(atom,
                                           atom_args,
                                           offset=True)
    response_offset = np.hstack([response, np.random.standard_normal((n, 1))])
    multinomial_lasso_offset.fit(X, response_offset)

    multinomial_lasso_weights = multinomial(atom,
                                            atom_args,
                                            case_weights=True)

    response_weights = np.hstack([response, np.ones((n, 1))])
    multinomial_lasso_weights.fit(X, response_weights)

    np.testing.assert_allclose(multinomial_lasso_weights._coefs,
                               multinomial_lasso._coefs)

    GridSearchCV(multinomial_lasso_offset, params, cv=3).fit(X, response_offset)
    GridSearchCV(multinomial_lasso_weights, params, cv=3).fit(X, response_weights)

    def atom_constructor(null_grad, **args):
        return group_lasso(**args)
    multinomial_lasso_lag = multinomial_lagrange(atom_constructor,
                                                 atom_args,
                                                 case_weights=True,
                                                 score_method='R2')
    GridSearchCV(multinomial_lasso_lag, params, cv=3).fit(X, response_weights)






import numpy as np

from ..block_maps import (block_columns, 
                          affine_transform,
                          block_rows)

def test_block_columns():

    X1 = np.random.standard_normal((20,10))
    X2 = np.random.standard_normal((23,10))

    B = block_columns([X1, X2])

    Z = np.random.standard_normal(B.input_shape)
    BL = B.linear_map(Z)
    BA = B.affine_map(Z)

    np.testing.assert_allclose(BL[:20], X1.dot(Z[:,0]))
    np.testing.assert_allclose(BL[20:], X2.dot(Z[:,1]))
    np.testing.assert_allclose(BA, BL)

    V = np.random.standard_normal(43)
    BTV = B.adjoint_map(V)

    np.testing.assert_allclose(np.column_stack([V[:20].dot(X1), V[20:].dot(X2)]), BTV)

def test_block_columns_affine():

    X1 = np.random.standard_normal((20,10))
    X2 = np.random.standard_normal((23,10))
    A1 = np.random.standard_normal(20)
    A2 = np.random.standard_normal(23)

    T1 = affine_transform(X1, A1)
    T2 = affine_transform(X2, A2)

    B = block_columns([T1, T2])

    Z = np.random.standard_normal(B.input_shape)
    BL = B.linear_map(Z)
    BA = B.affine_map(Z)

    np.testing.assert_allclose(BL[:20], X1.dot(Z[:,0]))
    np.testing.assert_allclose(BL[20:], X2.dot(Z[:,1]))
    #np.testing.assert_allclose(BA[:20], X1.dot(Z[:,0]) + A1)
    #np.testing.assert_allclose(BA[20:], X2.dot(Z[:,1]) + A2)
    np.testing.assert_allclose(BL + np.hstack([A1,A2]), BA)

    V = np.random.standard_normal(43)
    BTV = B.adjoint_map(V)

    np.testing.assert_allclose(np.column_stack([V[:20].dot(X1), V[20:].dot(X2)]), BTV)

def test_block_rows():

    X1 = np.random.standard_normal((20,10)).T
    X2 = np.random.standard_normal((23,10)).T

    B = block_rows([X1, X2])

    Z = np.random.standard_normal(B.input_shape)
    BL = B.linear_map(Z)
    BA = B.affine_map(Z)

    np.testing.assert_allclose(BL[:,0], X1.dot(Z[:20]))
    np.testing.assert_allclose(BL[:,1], X2.dot(Z[20:]))
    np.testing.assert_allclose(BA, BL)

    V = np.random.standard_normal(B.output_shape)
    BTV = B.adjoint_map(V)

    np.testing.assert_allclose(np.hstack([X1.T.dot(V[:,0]), X2.T.dot(V[:,1])]), BTV)

def test_block_rows_affine():

    X1 = np.random.standard_normal((20,10)).T
    X2 = np.random.standard_normal((23,10)).T
    A1 = np.random.standard_normal(10)
    A2 = np.random.standard_normal(10)

    T1 = affine_transform(X1, A1)
    T2 = affine_transform(X2, A2)

    B = block_rows([T1, T2])

    Z = np.random.standard_normal(B.input_shape)
    BL = B.linear_map(Z)
    BA = B.affine_map(Z)

    np.testing.assert_allclose(BL[:,0], X1.dot(Z[:20]))
    np.testing.assert_allclose(BL[:,1], X2.dot(Z[20:]))
    np.testing.assert_allclose(BA, BL + np.array([A1, A2]).T)

    V = np.random.standard_normal(B.output_shape)
    BTV = B.adjoint_map(V)

    np.testing.assert_allclose(np.hstack([X1.T.dot(V[:,0]), X2.T.dot(V[:,1])]), BTV)

